#define NODEFLEX 0

#define NF_PRECISION highp

#ifdef ENABLE_LIGHTING
#undef ENABLE_LIGHTING
#endif
#ifdef ENABLE_DIFFUSE_LIGHTING
#undef ENABLE_DIFFUSE_LIGHTING
#endif
#ifdef ENABLE_SPECULAR_LIGHTING
#undef ENABLE_SPECULAR_LIGHTING
#endif
#ifdef ENABLE_TONE_MAPPING
#undef ENABLE_TONE_MAPPING
#endif

#define ENABLE_LIGHTING
#define ENABLE_TONE_MAPPING
#define ENABLE_DIFFUSE_LIGHTING
#define ENABLE_SPECULAR_LIGHTING

#define SC_USE_USER_DEFINED_VERTEX_PROCESSING
#define SC_USE_USER_DEFINED_VS_MAIN
#define SC_SKIP_DUPLICATE_INCLUDES
#define SC_REMOVE_ENVMAP_FROM_AMBIENT_LIGHT_STRUCT

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>
#pragma once

#ifndef GLES20STD_TEXTURE
#define GLES20STD_TEXTURE

highp float depthToGlobal(highp float depth,highp vec2 projectionMatrixTerms){
	highp float m22=projectionMatrixTerms.x;
	highp float m32=projectionMatrixTerms.y;
	depth=depth*2.0 -1.0;
	return m32/(-depth -m22);
}

highp float depthToLocal(highp float depth,highp vec2 projectionMatrixTerms){
	highp float m22=projectionMatrixTerms.x;
	highp float m32=projectionMatrixTerms.y;
	depth=depth !=0.0 ? (-m22 -m32/depth) : 0.0;
	return depth*0.5+0.5;
}

#ifdef GL_ES
#if __VERSION__ >=300
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#define texture2DArray texture
#define texture3D texture
#define textureCube texture
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 0
#define SC_SUPPORT_3D_TEXTURE 0
#endif
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#endif

#define CLAMP_UV(uv,uvMin,uvMax,useClampToBorder,borderClampFactor){\
	float clampedUv=clamp(uv,uvMin,uvMax);\
\
	if (useClampToBorder){\
		borderClampFactor*=step(abs(uv -clampedUv),0.00001);\
	}\
\
	uv=clampedUv;\
}

#define SOFTWARE_WRAP(uv,softwareWrapMode,useClampToBorder,clampToBorderFactor){\
	if (softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE || softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER){\
		CLAMP_UV(uv,0.0,1.0,useClampToBorder,clampToBorderFactor);\
	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_REPEAT){\
		uv=fract(uv);\
	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_MIRRORED_REPEAT){\
		float uvFract=fract(uv);\
		float uvInt=uv -uvFract;\
		float uvOdd=step(0.25,fract(uvInt*0.5));\
		uv=mix(uvFract,1.0 -uvFract,clamp(uvOdd,0.0,1.0));\
	}\
}

#ifdef GL_ES
#define QUANTIZE_UV(A) A
#else
#define QUANTIZE_UV(A) A - (fract(A*4096.0)/4096.0);
#endif

#ifdef FRAGMENT_SHADER
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv,mipBias)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv,mipBias)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv,mipBias)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv,mipBias)
#else
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv)
#endif

#define SAMPLE_TEXTURE(tex,sampler,uvIn,useUvTransform,uvTransform,softwareWrapModeU,softwareWrapModeV,useUvMinMax,uvMinMax,useClampToBorder,borderColor,mipBias){\
	vec2 uvTemp=uvIn;\
\
	float clampToBorderFactor=1.0;\
\
	SOFTWARE_WRAP(uvTemp.x,softwareWrapModeU,useClampToBorder && !useUvMinMax,clampToBorderFactor);\
	SOFTWARE_WRAP(uvTemp.y,softwareWrapModeV,useClampToBorder && !useUvMinMax,clampToBorderFactor);\
\
	if (useUvMinMax){\
		CLAMP_UV(uvTemp.x,uvMinMax.x,uvMinMax.z,useClampToBorder,clampToBorderFactor);\
		CLAMP_UV(uvTemp.y,uvMinMax.y,uvMinMax.w,useClampToBorder,clampToBorderFactor);\
	}\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=TEXTURE2D(sampler,uvTemp,mipBias);\
\
	if (useClampToBorder){\
		tex=mix(borderColor,tex,clampToBorderFactor);\
	}\
}

#if SC_SUPPORT_2D_ARRAY_TEXTURE
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){\
	vec2 uvTemp=uvwIn.xy;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=TEXTURE2DARRAY(sampler,vec3(uvTemp,uvwIn.z),mipBias);\
}
#else
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){\
	tex=vec4(0.0,0.0,0.0,1.0);\
}
#endif

#if SC_SUPPORT_3D_TEXTURE
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){\
	vec3 uvwTemp=uvwIn;\
\
	uvwTemp=QUANTIZE_UV(uvwTemp);\
\
	tex=TEXTURE3D(sampler,uvwTemp,mipBias);\
}
#else
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){\
	tex=vec4(0.0,0.0,0.0,1.0);\
}
#endif

#define SAMPLE_TEXTURE_CUBE(tex,sampler,uvwIn,mipBias){\
	vec3 uvwTemp=uvwIn;\
\
	uvwTemp=QUANTIZE_UV(uvwTemp);\
\
	tex=TEXTURECUBE(sampler,uvwTemp,mipBias);\
}

#define SAMPLE_DEPTH_TEXTURE(tex,sampler,uvIn,useUvTransform,uvTransform){\
	vec2 uvTemp=uvIn;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	tex=texture2D(sampler,uvTemp).r;\
}

#define SAMPLE_DEPTH_TEXTURE_NDC(tex,sampler,uvIn,useUvTransform,uvTransform,projectionMatrixTerms){\
	vec2 uvTemp=uvIn;\
\
	if (useUvTransform){\
		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));\
	}\
\
	uvTemp=QUANTIZE_UV(uvTemp);\
\
	float depth=texture2D(sampler,uvTemp).r;\
\
	tex=depthToGlobal(depth,projectionMatrixTerms);\
}

#ifdef CUSTOM_PREPROCESSOR

#endif

#ifdef SC_TEXTURE_EXTERNAL
#define sc_SamplerExternalOES samplerExternalOES
#else
#define sc_SamplerExternalOES sampler2D
#endif

#ifdef FRAGMENT_SHADER

#if defined(GL_ES) && (__VERSION__ < 300) && !defined(GL_OES_standard_derivatives)
#define dFdx(A) (A)
#define dFdy(A) (A)
#define fwidth(A) (A)
#endif

#ifdef SC_TEXTURE_EXTERNAL
#define sc_textureExternalOESLod(tex,size,uv,lod) texture2D(tex,uv)

#else
vec4 sc_texture2DLod(sampler2D tex,vec2 textureSize,vec2 uv,float lod){
#if (__VERSION__==120)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return texture2DLod(tex,uv,lod);
#elif (__VERSION__ >=300)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	return texture2DLodEXT(tex,uv,lod);
#elif defined(GL_OES_standard_derivatives)
#define SC_TESTURE2DLOD_IS_SUPPORTED
	vec2 texels=uv*textureSize;
	float dudx=dFdx(texels.x);
	float dvdx=dFdx(texels.y);
	float dudy=dFdy(texels.x);
	float dvdy=dFdy(texels.y);
	float rho=max(length(vec2(dudx,dvdx)),length(vec2(dudy,dvdy)));
	float mu=max(abs(dudx),abs(dudy));
	float mv=max(abs(dvdx),abs(dvdy));
	float rho2=max(mu,mv);
	float mip=log2(rho2);
	float bias=lod -mip;
	return texture2D(tex,uv,bias);
#else
	return texture2D(tex,uv,lod);
#endif
}

#define sc_textureExternalOESLod(tex,size,uv,lod) sc_texture2DLod(tex,size,uv,lod)
#endif

#if SC_SUPPORT_2D_ARRAY_TEXTURE
vec4 sc_texture2DArrayLod(sc_FragmentPrecision sampler2DArray tex,vec3 uv,float lod){
#if (__VERSION__==120)
	return texture2DArrayLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return texture2DArrayLodEXT(tex,uv,lod);
#else
	return texture2DArray(tex,uv,lod);
#endif
}
#else

#define sc_texture2DArrayLod(tex,uv,lod) vec4(0,0,0,1)

#endif

#endif

#if SC_SUPPORT_3D_TEXTURE
vec4 sc_texture3DLod(sc_FragmentPrecision sampler3D tex,vec3 uv,float lod){
#if (__VERSION__==120)
	return texture3DLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return texture3DLodEXT(tex,uv,lod);
#else
	return texture3D(tex,uv,lod);
#endif
}
#else

#define sc_texture3DLod(tex,uv,lod) vec4(0,0,0,1)

#endif

vec4 sc_textureCubeLod(samplerCube tex,vec3 uv,float lod){
#if (__VERSION__ < 120)
	return textureCube(tex,uv);
#elif (__VERSION__==120)
	return textureCubeLod(tex,uv,lod);
#elif (__VERSION__ >=300)
	return textureLod(tex,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return textureCubeLodEXT(tex,uv,lod);
#else
	return textureCube(tex,uv,lod);
#endif
}

#endif

#define SCENARIUM

#ifdef GL_ES
#define MOBILE
#endif

#if SC_DEVICE_CLASS >=SC_DEVICE_CLASS_C && (!defined(MOBILE) || defined(GL_FRAGMENT_PRECISION_HIGH))
#define DEVICE_IS_FAST
#endif

#if defined(ENABLE_LIGHTING) || defined(ENABLE_EMISSIVE) || defined(ENABLE_VERTEX_COLOR_EMISSIVE) || defined(ENABLE_SIMPLE_REFLECTION) || defined(ENABLE_RIM_HIGHLIGHT) || defined(ENABLE_TONE_MAPPING)
#define SC_ENABLE_SRGB_EMULATION_IN_SHADER
#endif

varying vec4 varColor;

#ifndef UTILS_GLSL
#define UTILS_GLSL

#ifndef PI
#define PI 3.141592653589793238462643383279
#endif

#define SC_EPSILON 1e-6

#ifndef MAYA

float saturate(float value){
	return clamp(value,0.0,1.0);
}

vec2 saturate(vec2 value){
	return clamp(value,0.0,1.0);
}

vec3 saturate(vec3 value){
	return clamp(value,0.0,1.0);
}

#endif

#if defined(MAYA) || defined(SUBSTANCE)

float srgbToLinear(float x){
	return x <=0.04045 ? x*0.0773993808 : pow((x+0.055)/1.055,2.4);
}

float linearToSrgb(float x){
	return x <=0.0031308 ? 12.92*x : 1.055*pow(x,0.41666) -0.055;
}

#else

float srgbToLinear(float x){
#ifdef SC_ENABLE_SRGB_EMULATION_IN_SHADER
#ifdef DEVICE_IS_FAST
	return pow(x,2.2);
#else
	return x*x;
#endif
#else
	return x;
#endif
}

float linearToSrgb(float x){
#ifdef SC_ENABLE_SRGB_EMULATION_IN_SHADER
#ifdef DEVICE_IS_FAST
	return pow(x,1.0/2.2);
#else
	return sqrt(x);
#endif
#else
	return x;
#endif
}

#endif

vec3 srgbToLinear(vec3 color){
	return vec3(srgbToLinear(color.r),srgbToLinear(color.g),srgbToLinear(color.b));
}

vec3 linearToSrgb(vec3 color){
	return vec3(linearToSrgb(color.r),linearToSrgb(color.g),linearToSrgb(color.b));
}

#endif

#ifndef BLEND_MODES_GLSL
#define BLEND_MODES_GLSL

#include <std_fs.glsl>

#ifndef BLEND_MODES_EYECOLOR_GLSL
#define BLEND_MODES_EYECOLOR_GLSL

#ifndef RGBHSL_GLSL
#define RGBHSL_GLSL

vec3 RGBtoHCV(vec3 rgb)
{
	vec4 p=(rgb.g < rgb.b) ? vec4(rgb.bg,-1.0,2.0/3.0) : vec4(rgb.gb,0.0,-1.0/3.0);
	vec4 q=(rgb.r < p.x) ? vec4(p.xyw,rgb.r) : vec4(rgb.r,p.yzx);

	float c=q.x -min(q.w,q.y);
	float h=abs((q.w -q.y)/(6.0*c+1e-7)+q.z);
	float v=q.x;

	return vec3(h,c,v);
}

vec3 RGBToHSL(vec3 rgb)
{
	vec3 hcv=RGBtoHCV(rgb);

	float lum=hcv.z -hcv.y*0.5;
	float sat=hcv.y/(1.0 -abs(2.0*lum -1.0)+1e-7);

	return vec3(hcv.x,sat,lum);
}

vec3 HUEtoRGB(float hue)
{
	float r=abs(6.0*hue -3.0) -1.0;
	float g=2.0 -abs(6.0*hue -2.0);
	float b=2.0 -abs(6.0*hue -4.0);
	return clamp(vec3(r,g,b),0.0,1.0);
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb=HUEtoRGB(hsl.x);
	float c=(1.0 -abs(2.0*hsl.z -1.0))*hsl.y;
	rgb=(rgb -0.5)*c+hsl.z;
	return rgb;
}

#endif

#ifdef BLEND_MODE_REALISTIC
#define COLOR_MODE 0
#endif

#ifdef BLEND_MODE_DIVISION
#define COLOR_MODE 1
#endif

#ifdef BLEND_MODE_BRIGHT
#define COLOR_MODE 2
#endif

#ifdef BLEND_MODE_FORGRAY
#define COLOR_MODE 3
#endif

#ifdef BLEND_MODE_NOTBRIGHT
#define COLOR_MODE 4
#endif

#ifdef BLEND_MODE_INTENSE
#define COLOR_MODE 5
#endif

#ifdef COLOR_MODE

uniform float	correctedIntensity;
uniform sampler2D intensityTexture;

#if COLOR_MODE==0 || COLOR_MODE==3 || COLOR_MODE==4

float transformSingleColor(float original,float intMap,float target){
	return original/pow((1.0 -target),intMap);
}

#endif
#if COLOR_MODE==1

float transformSingleColor(float original,float intMap,float target){
	return original/(1.0 -target);
}

#endif
#if COLOR_MODE==2

float transformSingleColor(float original,float intMap,float target){
	return original/pow((1.0 -target),2.0 -2.0*original);
}

#endif

#if COLOR_MODE !=5

vec3 transformColor(float yValue,vec3 original,vec3 target,float weight,float intMap){
	vec3 tmpColor;
	tmpColor.r=transformSingleColor(yValue,intMap,target.r);
	tmpColor.g=transformSingleColor(yValue,intMap,target.g);
	tmpColor.b=transformSingleColor(yValue,intMap,target.b);
	tmpColor=clamp(tmpColor,0.0,1.0);
	vec3 resColor=mix(original,tmpColor,weight);
	return resColor;
}

#endif

#if COLOR_MODE==5

vec3 transformColor(float yValue,vec3 original,vec3 target,float weight,float intMap){
	vec3 hslOrig=RGBToHSL(original);
	vec3 res;
	res.r=target.r;
	res.g=target.g;
	res.b=hslOrig.b;
	res=HSLToRGB(res);
	vec3 resColor=mix(original,res,weight);
	return resColor;
}

#endif

float unpack1(float inp,float mul){
	return inp*mul;
}

float unpack2(vec2 inp,float mul){
	return (inp[0]*256.0+inp[1])/257.0*mul;
}

float unpack3(vec3 inp,float mul){

	return (inp[0]*256.0+inp[1]+inp[2]/256.0)/(256.0+1.0+1.0/256.0)*mul;
}

float getYValue(vec3 rgb){
	return 0.299*rgb.r+0.587*rgb.g+0.114*rgb.b;
}

vec3 eyeColorBlend(vec3 texColor,vec3 resColor)
{
	float newYValue=getYValue(texColor);

	float weight=1.0;
	float fragmentCorrectedIntensity=pow(newYValue,1.0/correctedIntensity);
	vec3 intenseMapCompressed=texture2D(intensityTexture,vec2(fragmentCorrectedIntensity,0.5)).rgb;
	float intenseMapValue=unpack3(intenseMapCompressed,16.0);

#if COLOR_MODE==3
	intenseMapValue=max(intenseMapValue,1.0);
#endif
#if COLOR_MODE==4
	intenseMapValue=min(intenseMapValue,1.0);
#endif

	vec3 newColor=transformColor(newYValue,texColor,resColor,weight,intenseMapValue);
	return newColor;
}

#define definedBlend eyeColorBlend

#endif

#endif

#ifdef FRAGMENT_SHADER
#ifdef sc_BlendMode_Custom

vec3 ContrastSaturationBrightness(vec3 color,float brt,float sat,float con)
{

	const float AvgLumR=0.5;
	const float AvgLumG=0.5;
	const float AvgLumB=0.5;

	const vec3 LumCoeff=vec3(0.2125,0.7154,0.0721);

	vec3 AvgLumin=vec3(AvgLumR,AvgLumG,AvgLumB);
	vec3 brtColor=color*brt;
	vec3 intensity=vec3(dot(brtColor,LumCoeff));
	vec3 satColor=mix(intensity,brtColor,sat);
	vec3 conColor=mix(AvgLumin,satColor,con);
	return conColor;
}

float BlendAddf(float base,float blend){
	return min(base+blend,1.0);
}
float BlendSubtractf(float base,float blend){
	return max(base+blend -1.0,0.0);
}
float BlendLinearDodgef(float base,float blend){
	return min(base+blend,1.0);
}
float BlendLinearBurnf(float base,float blend){
	return max(base+blend -1.0,0.0);
}
float BlendLightenf(float base,float blend){
	return max(blend,base);
}
float BlendDarkenf(float base,float blend){
	return min(blend,base);
}
float BlendScreenf(float base,float blend){
	return (1.0 - ((1.0 - (base))*(1.0 - (blend))));
}
float BlendOverlayf(float base,float blend){
	return (base < 0.5 ? (2.0*(base)*(blend)) : (1.0 -2.0*(1.0 - (base))*(1.0 - (blend))));
}
float BlendSoftLightf(float base,float blend){
	return ((1.0 -2.0*(blend))*(base)*(base)+2.0*(base)*(blend));
}
float BlendColorDodgef(float base,float blend){
	return ((blend==1.0) ? blend : min((base)/(1.0 - (blend)),1.0));
}
float BlendColorBurnf(float base,float blend){
	return ((blend==0.0) ? blend : max((1.0 - ((1.0 - (base))/(blend))),0.0));
}
float BlendLinearLightf(float base,float blend){
	if(blend < 0.5){
		return BlendLinearBurnf(base,2.0*blend);
	}
	else {
		return BlendLinearDodgef(base,2.0*(blend -0.5));
	}
}
float BlendVividLightf(float base,float blend){
	if(blend < 0.5){
		return BlendColorBurnf(base,2.0*blend);
	}
	else {
		return BlendColorDodgef(base,2.0*(blend -0.5));
	}
}
float BlendPinLightf(float base,float blend){
	if(blend < 0.5){
		return BlendDarkenf(base,2.0*blend);
	}
	else {
		return BlendLightenf(base,2.0*(blend -0.5));
	}
}
float BlendHardMixf(float base,float blend){
	if(BlendVividLightf(base,blend) < 0.5){
		return 0.0;
	}
	else {
		return 1.0;
	}
}
float BlendReflectf(float base,float blend){
	return ((blend==1.0) ? blend : min((base)*(base)/(1.0 - (blend)),1.0));
}

#define BlendNormal(base,blend) 		(blend)
#define BlendLighten(base,blend)		(vec3(BlendLightenf(base.r,blend.r),BlendLightenf(base.g,blend.g),BlendLightenf(base.b,blend.b)))
#define BlendDarken(base,blend)		(vec3(BlendDarkenf(base.r,blend.r),BlendDarkenf(base.g,blend.g),BlendDarkenf(base.b,blend.b)))
#define BlendMultiply(base,blend) 		((base)*(blend))
#define BlendDivide(base,blend) 		((blend)/(base))
#define BlendAverage(base,blend) 		((base+blend)/2.0)
#define BlendAdd(base,blend) 			min(base+blend,vec3(1.0))
#define BlendSubtract(base,blend) 		max(base+blend -vec3(1.0),vec3(0.0))
#define BlendDifference(base,blend) 	abs(base - (blend))
#define BlendNegation(base,blend) 		(vec3(1.0) -abs(vec3(1.0) - (base) - (blend)))
#define BlendExclusion(base,blend) 	(base+blend -2.0*(base)*(blend))
#define BlendScreen(base,blend) 		vec3(BlendScreenf(base.r,blend.r),BlendScreenf(base.g,blend.g),BlendScreenf(base.b,blend.b))

#define BlendOverlay(base,blend) 		vec3(BlendOverlayf(base.r,blend.r),BlendOverlayf(base.g,blend.g),BlendOverlayf(base.b,blend.b))
#define BlendSoftLight(base,blend) 	vec3(BlendSoftLightf(base.r,blend.r),BlendSoftLightf(base.g,blend.g),BlendSoftLightf(base.b,blend.b))
#define BlendHardLight(base,blend) 	BlendOverlay(blend,base)
#define BlendColorDodge(base,blend) 	vec3(BlendColorDodgef(base.r,blend.r),BlendColorDodgef(base.g,blend.g),BlendColorDodgef(base.b,blend.b))
#define BlendColorBurn(base,blend) 	vec3(BlendColorBurnf(base.r,blend.r),BlendColorBurnf(base.g,blend.g),BlendColorBurnf(base.b,blend.b))
#define BlendLinearDodge(base,blend)	BlendAdd(base,blend)
#define BlendLinearBurn(base,blend)	BlendSubtract(base,blend)

#define BlendLinearLight(base,blend) 	vec3(BlendLinearLightf(base.r,blend.r),BlendLinearLightf(base.g,blend.g),BlendLinearLightf(base.b,blend.b))
#define BlendVividLight(base,blend) 	vec3(BlendVividLightf(base.r,blend.r),BlendVividLightf(base.g,blend.g),BlendVividLightf(base.b,blend.b))
#define BlendPinLight(base,blend) 		vec3(BlendPinLightf(base.r,blend.r),BlendPinLightf(base.g,blend.g),BlendPinLightf(base.b,blend.b))
#define BlendHardMix(base,blend) 		vec3(BlendHardMixf(base.r,blend.r),BlendHardMixf(base.g,blend.g),BlendHardMixf(base.b,blend.b))
#define BlendReflect(base,blend) 		vec3(BlendReflectf(base.r,blend.r),BlendReflectf(base.g,blend.g),BlendReflectf(base.b,blend.b))
#define BlendGlow(base,blend) 			BlendReflect(blend,base)
#define BlendPhoenix(base,blend) 		(min(base,blend) -max(base,blend)+vec3(1.0))
#define BlendOpacity(base,blend,F,O) 	(F(base,blend)*O+(blend)*(1.0 -O))

vec3 BlendHue(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(RGBToHSL(blend).r,baseHSL.g,baseHSL.b));
}

vec3 BlendSaturation(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r,RGBToHSL(blend).g,baseHSL.b));
}

vec3 BlendColor(vec3 base,vec3 blend)
{
	vec3 blendHSL=RGBToHSL(blend);
	return HSLToRGB(vec3(blendHSL.r,blendHSL.g,RGBToHSL(base).b));
}

vec3 BlendLuminosity(vec3 base,vec3 blend)
{
	vec3 baseHSL=RGBToHSL(base);
	return HSLToRGB(vec3(baseHSL.r,baseHSL.g,RGBToHSL(blend).b));
}

#define GammaCorrection(color,gamma)								pow(color,1.0/gamma)

#define LevelsControlInputRange(color,minInput,maxInput)				min(max(color -vec3(minInput),vec3(0.0))/(vec3(maxInput) -vec3(minInput)),vec3(1.0))
#define LevelsControlInput(color,minInput,gamma,maxInput)				GammaCorrection(LevelsControlInputRange(color,minInput,maxInput),gamma)
#define LevelsControlOutputRange(color,minOutput,maxOutput) 			mix(vec3(minOutput),vec3(maxOutput),color)
#define LevelsControl(color,minInput,gamma,maxInput,minOutput,maxOutput) 	LevelsControlOutputRange(LevelsControlInput(color,minInput,gamma,maxInput),minOutput,maxOutput)

#if defined BLEND_MODE_NORMAL
#define definedBlend(a,b) BlendNormal(a,b)

#elif defined BLEND_MODE_LIGHTEN
#define definedBlend(a,b) BlendLighten(a,b)

#elif defined BLEND_MODE_DARKEN
#define definedBlend(a,b) BlendDarken(a,b)

#elif defined BLEND_MODE_MULTIPLY
#define definedBlend(a,b) BlendMultiply(a,b)

#elif defined BLEND_MODE_DIVIDE
#define definedBlend(a,b) BlendDivide(a,b)

#elif defined BLEND_MODE_AVERAGE
#define definedBlend(a,b) BlendAverage(a,b)

#elif defined BLEND_MODE_ADD
#define definedBlend(a,b) BlendAdd(a,b)

#elif defined BLEND_MODE_SUBTRACT
#define definedBlend(a,b) BlendSubtract(a,b)

#elif defined BLEND_MODE_DIFFERENCE
#define definedBlend(a,b) BlendDifference(a,b)

#elif defined BLEND_MODE_NEGATION
#define definedBlend(a,b) BlendNegation(a,b)

#elif defined BLEND_MODE_EXCLUSION
#define definedBlend(a,b) BlendExclusion(a,b)

#elif defined BLEND_MODE_SCREEN
#define definedBlend(a,b) BlendScreen(a,b)

#elif defined BLEND_MODE_OVERLAY
#define definedBlend(a,b) BlendOverlay(a,b)

#elif defined BLEND_MODE_SOFT_LIGHT
#define definedBlend(a,b) BlendSoftLight(a,b)

#elif defined BLEND_MODE_HARD_LIGHT
#define definedBlend(a,b) BlendHardLight(a,b)

#elif defined BLEND_MODE_COLOR_DODGE
#define definedBlend(a,b) BlendColorDodge(a,b)

#elif defined BLEND_MODE_COLOR_BURN
#define definedBlend(a,b) BlendColorBurn(a,b)

#elif defined BLEND_MODE_LINEAR_LIGHT
#define definedBlend(a,b) BlendLinearLight(a,b)

#elif defined BLEND_MODE_VIVID_LIGHT
#define definedBlend(a,b) BlendVividLight(a,b)

#elif defined BLEND_MODE_PIN_LIGHT
#define definedBlend(a,b) BlendPinLight(a,b)

#elif defined BLEND_MODE_HARD_MIX
#define definedBlend(a,b) BlendHardMix(a,b)

#elif defined BLEND_MODE_HARD_REFLECT
#define definedBlend(a,b) BlendReflect(a,b)

#elif defined BLEND_MODE_HARD_GLOW
#define definedBlend(a,b) BlendGlow(a,b)

#elif defined BLEND_MODE_HARD_PHOENIX
#define definedBlend(a,b) BlendPhoenix(a,b)

#elif defined BLEND_MODE_HUE
#define definedBlend(a,b) BlendHue(a,b)

#elif defined BLEND_MODE_SATURATION
#define definedBlend(a,b) BlendSaturation(a,b)

#elif defined BLEND_MODE_COLOR
#define definedBlend(a,b) BlendColor(a,b)

#elif defined BLEND_MODE_LUMINOSITY
#define definedBlend(a,b) BlendLuminosity(a,b)

#endif

#ifndef definedBlend
#error If you define sc_BlendMode_Custom,you must also define a BLEND_MODE_*!
#endif

vec4 applyCustomBlend(vec4 color){
	vec4 result;
	vec3 framebuffer=getFramebufferColor().rgb;
	result.rgb=definedBlend(framebuffer,color.rgb);
	result.rgb=mix(framebuffer,result.rgb,color.a);
	result.a=1.0;
	return result;
}

#endif
#endif

#endif

#if !defined(MOBILE) && !NODEFLEX
#define STUDIO
#endif

#if defined(STUDIO)
#define ssConstOrUniformPrecision	uniform NF_PRECISION
#define ssConstOrUniform			uniform
#else
#define ssConstOrUniformPrecision const
#define ssConstOrUniform			const
#endif

#if defined(STUDIO)
#define ssPortConstant(xValue,xUniform)	xUniform
#else
#define ssPortConstant(xValue,xUniform)	xValue
#endif

#define float2 vec2
#define float3 vec3
#define float4 vec4
#define bool2	bvec2
#define bool3	bvec3
#define bool4	bvec4
#define float2x2 mat2
#define float3x3 mat3
#define float4x4 mat4

mat4 ssTranspose(mat4 Matrix)
{
	vec4 i0=Matrix[0];
	vec4 i1=Matrix[1];
	vec4 i2=Matrix[2];
	vec4 i3=Matrix[3];

	return mat4(vec4(i0.x,i1.x,i2.x,i3.x),vec4(i0.y,i1.y,i2.y,i3.y),vec4(i0.z,i1.z,i2.z,i3.z),vec4(i0.w,i1.w,i2.w,i3.w));
}

mat3 ssTranspose(mat3 Matrix)
{
	vec3 i0=Matrix[0];
	vec3 i1=Matrix[1];
	vec3 i2=Matrix[2];

	return mat3(vec3(i0.x,i1.x,i2.x),vec3(i0.y,i1.y,i2.y),vec3(i0.z,i1.z,i2.z));
}

mat2 ssTranspose(mat2 Matrix)
{
	vec2 i0=Matrix[0];
	vec2 i1=Matrix[1];

	return mat2(vec2(i0.x,i1.x),vec2(i0.y,i1.y));
}

float ssConditional(float C,float A,float B){return (bool(C*1.0)) ? A : B;}
float2 ssConditional(float2 C,float A,float2 B){return float2(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y));}
float3 ssConditional(float3 C,float A,float3 B){return float3(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y),ssConditional(C.z,A,B.z));}
float4 ssConditional(float4 C,float A,float4 B){return float4(ssConditional(C.x,A,B.x),ssConditional(C.y,A,B.y),ssConditional(C.z,A,B.z),ssConditional(C.w,A,B.w));}
float2 ssConditional(float2 C,float A,float B){return float2(ssConditional(C.x,A,B),ssConditional(C.y,A,B));}
float3 ssConditional(float3 C,float A,float B){return float3(ssConditional(C.x,A,B),ssConditional(C.y,A,B),ssConditional(C.z,A,B));}
float4 ssConditional(float4 C,float A,float B){return float4(ssConditional(C.x,A,B),ssConditional(C.y,A,B),ssConditional(C.z,A,B),ssConditional(C.w,A,B));}
float2 ssConditional(float C,float A,float2 B){return float2(ssConditional(C,A,B.x),ssConditional(C,A,B.y));}
float3 ssConditional(float C,float A,float3 B){return float3(ssConditional(C,A,B.x),ssConditional(C,A,B.y),ssConditional(C,A,B.z));}
float4 ssConditional(float C,float A,float4 B){return float4(ssConditional(C,A,B.x),ssConditional(C,A,B.y),ssConditional(C,A,B.z),ssConditional(C,A,B.w));}

float ssEqual(float A,float B){return (A==B) ? 1.0 : 0.0;}
float2 ssEqual(float2 A,float2 B){return float2(ssEqual(A.x,B.x),ssEqual(A.y,B.y));}
float3 ssEqual(float3 A,float3 B){return float3(ssEqual(A.x,B.x),ssEqual(A.y,B.y),ssEqual(A.z,B.z));}
float4 ssEqual(float4 A,float4 B){return float4(ssEqual(A.x,B.x),ssEqual(A.y,B.y),ssEqual(A.z,B.z),ssEqual(A.w,B.w));}
float2 ssEqual(float2 A,float B){return float2(ssEqual(A.x,B),ssEqual(A.y,B));}
float3 ssEqual(float3 A,float B){return float3(ssEqual(A.x,B),ssEqual(A.y,B),ssEqual(A.z,B));}
float4 ssEqual(float4 A,float B){return float4(ssEqual(A.x,B),ssEqual(A.y,B),ssEqual(A.z,B),ssEqual(A.w,B));}
float2 ssEqual(float A,float2 B){return float2(ssEqual(A,B.x),ssEqual(A,B.y));}
float3 ssEqual(float A,float3 B){return float3(ssEqual(A,B.x),ssEqual(A,B.y),ssEqual(A,B.z));}
float4 ssEqual(float A,float4 B){return float4(ssEqual(A,B.x),ssEqual(A,B.y),ssEqual(A,B.z),ssEqual(A,B.w));}

float ssNotEqual(float A,float B){return (A==B) ? 0.0 : 1.0;}
float2 ssNotEqual(float2 A,float2 B){return float2(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y));}
float3 ssNotEqual(float3 A,float3 B){return float3(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y),ssNotEqual(A.z,B.z));}
float4 ssNotEqual(float4 A,float4 B){return float4(ssNotEqual(A.x,B.x),ssNotEqual(A.y,B.y),ssNotEqual(A.z,B.z),ssNotEqual(A.w,B.w));}
float2 ssNotEqual(float2 A,float B){return float2(ssNotEqual(A.x,B),ssNotEqual(A.y,B));}
float3 ssNotEqual(float3 A,float B){return float3(ssNotEqual(A.x,B),ssNotEqual(A.y,B),ssNotEqual(A.z,B));}
float4 ssNotEqual(float4 A,float B){return float4(ssNotEqual(A.x,B),ssNotEqual(A.y,B),ssNotEqual(A.z,B),ssNotEqual(A.w,B));}
float2 ssNotEqual(float A,float2 B){return float2(ssNotEqual(A,B.x),ssNotEqual(A,B.y));}
float3 ssNotEqual(float A,float3 B){return float3(ssNotEqual(A,B.x),ssNotEqual(A,B.y),ssNotEqual(A,B.z));}
float4 ssNotEqual(float A,float4 B){return float4(ssNotEqual(A,B.x),ssNotEqual(A,B.y),ssNotEqual(A,B.z),ssNotEqual(A,B.w));}

float ssLarger(float A,float B){return (A > B) ? 1.0 : 0.0;}
float2 ssLarger(float2 A,float2 B){return float2(ssLarger(A.x,B.x),ssLarger(A.y,B.y));}
float3 ssLarger(float3 A,float3 B){return float3(ssLarger(A.x,B.x),ssLarger(A.y,B.y),ssLarger(A.z,B.z));}
float4 ssLarger(float4 A,float4 B){return float4(ssLarger(A.x,B.x),ssLarger(A.y,B.y),ssLarger(A.z,B.z),ssLarger(A.w,B.w));}
float2 ssLarger(float2 A,float B){return float2(ssLarger(A.x,B),ssLarger(A.y,B));}
float3 ssLarger(float3 A,float B){return float3(ssLarger(A.x,B),ssLarger(A.y,B),ssLarger(A.z,B));}
float4 ssLarger(float4 A,float B){return float4(ssLarger(A.x,B),ssLarger(A.y,B),ssLarger(A.z,B),ssLarger(A.w,B));}
float2 ssLarger(float A,float2 B){return float2(ssLarger(A,B.x),ssLarger(A,B.y));}
float3 ssLarger(float A,float3 B){return float3(ssLarger(A,B.x),ssLarger(A,B.y),ssLarger(A,B.z));}
float4 ssLarger(float A,float4 B){return float4(ssLarger(A,B.x),ssLarger(A,B.y),ssLarger(A,B.z),ssLarger(A,B.w));}

float ssLargerOrEqual(float A,float B){return (A >=B) ? 1.0 : 0.0;}
float2 ssLargerOrEqual(float2 A,float2 B){return float2(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y));}
float3 ssLargerOrEqual(float3 A,float3 B){return float3(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y),ssLargerOrEqual(A.z,B.z));}
float4 ssLargerOrEqual(float4 A,float4 B){return float4(ssLargerOrEqual(A.x,B.x),ssLargerOrEqual(A.y,B.y),ssLargerOrEqual(A.z,B.z),ssLargerOrEqual(A.w,B.w));}
float2 ssLargerOrEqual(float2 A,float B){return float2(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B));}
float3 ssLargerOrEqual(float3 A,float B){return float3(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B),ssLargerOrEqual(A.z,B));}
float4 ssLargerOrEqual(float4 A,float B){return float4(ssLargerOrEqual(A.x,B),ssLargerOrEqual(A.y,B),ssLargerOrEqual(A.z,B),ssLargerOrEqual(A.w,B));}
float2 ssLargerOrEqual(float A,float2 B){return float2(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y));}
float3 ssLargerOrEqual(float A,float3 B){return float3(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y),ssLargerOrEqual(A,B.z));}
float4 ssLargerOrEqual(float A,float4 B){return float4(ssLargerOrEqual(A,B.x),ssLargerOrEqual(A,B.y),ssLargerOrEqual(A,B.z),ssLargerOrEqual(A,B.w));}

float ssSmaller(float A,float B){return (A < B) ? 1.0 : 0.0;}
float2 ssSmaller(float2 A,float2 B){return float2(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y));}
float3 ssSmaller(float3 A,float3 B){return float3(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y),ssSmaller(A.z,B.z));}
float4 ssSmaller(float4 A,float4 B){return float4(ssSmaller(A.x,B.x),ssSmaller(A.y,B.y),ssSmaller(A.z,B.z),ssSmaller(A.w,B.w));}
float2 ssSmaller(float2 A,float B){return float2(ssSmaller(A.x,B),ssSmaller(A.y,B));}
float3 ssSmaller(float3 A,float B){return float3(ssSmaller(A.x,B),ssSmaller(A.y,B),ssSmaller(A.z,B));}
float4 ssSmaller(float4 A,float B){return float4(ssSmaller(A.x,B),ssSmaller(A.y,B),ssSmaller(A.z,B),ssSmaller(A.w,B));}
float2 ssSmaller(float A,float2 B){return float2(ssSmaller(A,B.x),ssSmaller(A,B.y));}
float3 ssSmaller(float A,float3 B){return float3(ssSmaller(A,B.x),ssSmaller(A,B.y),ssSmaller(A,B.z));}
float4 ssSmaller(float A,float4 B){return float4(ssSmaller(A,B.x),ssSmaller(A,B.y),ssSmaller(A,B.z),ssSmaller(A,B.w));}

float ssSmallerOrEqual(float A,float B){return (A <=B) ? 1.0 : 0.0;}
float2 ssSmallerOrEqual(float2 A,float2 B){return float2(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y));}
float3 ssSmallerOrEqual(float3 A,float3 B){return float3(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y),ssSmallerOrEqual(A.z,B.z));}
float4 ssSmallerOrEqual(float4 A,float4 B){return float4(ssSmallerOrEqual(A.x,B.x),ssSmallerOrEqual(A.y,B.y),ssSmallerOrEqual(A.z,B.z),ssSmallerOrEqual(A.w,B.w));}
float2 ssSmallerOrEqual(float2 A,float B){return float2(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B));}
float3 ssSmallerOrEqual(float3 A,float B){return float3(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B),ssSmallerOrEqual(A.z,B));}
float4 ssSmallerOrEqual(float4 A,float B){return float4(ssSmallerOrEqual(A.x,B),ssSmallerOrEqual(A.y,B),ssSmallerOrEqual(A.z,B),ssSmallerOrEqual(A.w,B));}
float2 ssSmallerOrEqual(float A,float2 B){return float2(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y));}
float3 ssSmallerOrEqual(float A,float3 B){return float3(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y),ssSmallerOrEqual(A,B.z));}
float4 ssSmallerOrEqual(float A,float4 B){return float4(ssSmallerOrEqual(A,B.x),ssSmallerOrEqual(A,B.y),ssSmallerOrEqual(A,B.z),ssSmallerOrEqual(A,B.w));}

float ssAll(float A){return (bool(A*1.0)) ? 1.0 : 0.0;}
float ssAll(float2 A){return all(notEqual(A,float2(0.0))) ? 1.0 : 0.0;}
float ssAll(float3 A){return all(notEqual(A,float3(0.0))) ? 1.0 : 0.0;}
float ssAll(float4 A){return all(notEqual(A,float4(0.0))) ? 1.0 : 0.0;}

float ssAny(float A){return (bool(A*1.0)) ? 1.0 : 0.0;}
float ssAny(float2 A){return any(notEqual(A,float2(0.0))) ? 1.0 : 0.0;}
float ssAny(float3 A)	{return any(notEqual(A,float3(0.0))) ? 1.0 : 0.0;}
float ssAny(float4 A)	{return any(notEqual(A,float4(0.0))) ? 1.0 : 0.0;}

float ssNot(float A){return (bool(A*1.0)) ? 0.0 : 1.0;}
float2 ssNot(float2 A){return float2(ssNot(A.x),ssNot(A.y));}
float3 ssNot(float3 A){return float3(ssNot(A.x),ssNot(A.y),ssNot(A.z));}
float4 ssNot(float4 A){return float4(ssNot(A.x),ssNot(A.y),ssNot(A.z),ssNot(A.w));}

float tempSrgbToLinear(float x)
{
	#if defined(DEVICE_IS_FAST)
	return pow(x,2.2);
	#else
	return x*x;
	#endif
}

float ssSRGB_to_Linear(float A){return tempSrgbToLinear(A);}
float2 ssSRGB_to_Linear(float2 A){return float2(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g));}
float3 ssSRGB_to_Linear(float3 A){return float3(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g),tempSrgbToLinear(A.b));}
float4 ssSRGB_to_Linear(float4 A){return float4(tempSrgbToLinear(A.r),tempSrgbToLinear(A.g),tempSrgbToLinear(A.b),A.a);}

float tempLinearToSrgb(float x)
{
	#if defined(DEVICE_IS_FAST)
	return pow(x,1.0/2.2);
	#else
	return sqrt(x);
	#endif
}

float ssLinear_to_SRGB(float A){return tempLinearToSrgb(A);}
float2 ssLinear_to_SRGB(float2 A){return float2(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g));}
float3 ssLinear_to_SRGB(float3 A){return float3(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g),tempLinearToSrgb(A.b));}
float4 ssLinear_to_SRGB(float4 A){return float4(tempLinearToSrgb(A.r),tempLinearToSrgb(A.g),tempLinearToSrgb(A.b),A.a);}

float Dummy1;
float2 Dummy2;
float3 Dummy3;
float4 Dummy4;

#if defined(STUDIO)

struct ssPreviewInfo
{
	float4 Color;
	bool Saved;
};

uniform NF_PRECISION int PreviewEnabled;
uniform NF_PRECISION int PreviewNodeID;

varying float4 PreviewVertexColor;
varying float PreviewVertexSaved;

#define ssDisableVertexChanges 					(PreviewEnabled==1)

#define ssFunctionDefs 							ssGlobals Globals,inout ssPreviewInfo PreviewInfo
#define ssFunctionInputs 						Globals,PreviewInfo

#define ssSetupPreview_Vertex					ssPreviewInfo PreviewInfo;PreviewInfo.Color=PreviewVertexColor=float4(0.5);PreviewInfo.Saved=false;PreviewVertexSaved=0.0;
#define ssSetupPreview_Pixel					ssPreviewInfo PreviewInfo;PreviewInfo.Color=PreviewVertexColor;PreviewInfo.Saved=(bool(PreviewVertexSaved*1.0)) ? true : false;
#define ssPreviewSave(xCode,xNodeID,xAlpha) if (PreviewEnabled==1 && !PreviewInfo.Saved && xNodeID==PreviewNodeID){PreviewInfo.Saved=true;{PreviewInfo.Color=xCode;if (!xAlpha) PreviewInfo.Color.a=1.0;}}
#define ssPreviewForceSave(xCode) 			if (PreviewEnabled==0){PreviewInfo.Saved=true;{PreviewInfo.Color=xCode;}}
#define ssPreviewOutput_Vertex					if (PreviewInfo.Saved){PreviewVertexColor=float4(PreviewInfo.Color.rgb,1.0);PreviewVertexSaved=1.0;}
#define ssPreviewOutput_Pixel					if (PreviewEnabled==1){if (PreviewInfo.Saved){FinalColor=float4(PreviewInfo.Color);} else {FinalColor=vec4(0.0,0.0,0.0,0.0);}}

#else

#define ssDisableVertexChanges				false

#define ssFunctionDefs 						ssGlobals Globals
#define ssFunctionInputs 					Globals

#define ssSetupPreview_Vertex
#define ssSetupPreview_Pixel
#define ssPreviewSave(xCode,xNodeID,xAlpha)
#define ssPreviewForceSave(xCode)
#define ssPreviewOutput_Vertex
#define ssPreviewOutput_Pixel

#endif

#ifdef FRAGMENT_SHADER
#ifdef GL_FRAGMENT_PRECISION_HIGH
#define ssTimeElapsed 	float(sc_TimeElapsed)
#else
#define ssTimeElapsed 	Globals.TimeElapsed
#endif
#else
#define ssTimeElapsed 		float(sc_TimeElapsed)
#endif

#ifdef FRAGMENT_SHADER
#ifdef GL_FRAGMENT_PRECISION_HIGH
#define ssTimeDelta 	float(sc_TimeDelta)
#else
#define ssTimeDelta		Globals.TimeDelta
#endif
#else
#define ssTimeDelta			float(sc_TimeDelta)
#endif

#ifdef VERTEX_SHADER

attribute vec4 color;

void ngsVertexShaderBegin(out sc_Vertex_t v)
{
	v=sc_LoadVertexAttributes();

	sc_BlendVertex(v);
	sc_SkinVertex(v);

	#if (sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN)

	varPos	=vec3(0.0);
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN_MV

	varPos	=vec3(0.0);
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_WORLD

	varPos	=v.position.xyz;
	varNormal=v.normal;
	varTangent=v.tangent;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_OBJECT

	varPos	=(sc_ModelMatrix*v.position).xyz;
	varNormal=sc_NormalMatrix*v.normal;
	varTangent=sc_NormalMatrix*v.tangent;

	#else

	#error Unknown sc_RenderingSpace!

	#endif

	varColor=color;
}

void ngsVertexShaderEnd(inout sc_Vertex_t v,vec3 WorldPosition,vec3 WorldNormal,vec3 WorldTangent,vec4 ScreenPosition)
{
	varPos	=WorldPosition;
	varNormal=WorldNormal;
	varTangent=WorldTangent;

	#if (sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN)

	float4 screenPosition=ScreenPosition;

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_SCREEN_MV

	float4 screenPosition=(sc_ModelViewMatrix*v.position)*vec4(1.0/sc_Camera.aspect,1.0,1.0,1.0);

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_WORLD

	float4 screenPosition=sc_ViewProjectionMatrix*float4(varPos.xyz,1.0);

	#elif sc_RenderingSpace==SC_RENDERING_SPACE_OBJECT

	float4 screenPosition=sc_ViewProjectionMatrix*float4(varPos.xyz,1.0);

	#else

	#error Unknown sc_RenderingSpace!

	#endif

	varNormal		=normalize(varNormal);
	varTangent		=normalize(varTangent);
	varBitangentSign	=tangent.w;
	varTex0			=v.texture0;
	varTex1			=v.texture1;
	varScreenPos		=screenPosition;
	varScreenTexturePos=screenPosition.xy;

	#if defined(sc_ProjectiveShadowsReceiver)

	varShadowTex=getProjectedTexCoords(v.position);

	#endif

	#if (sc_DepthBufferMode==SC_DEPTH_BUFFER_MODE_LOGARITHMIC)

	float fCoefficient=2.0/log2(sc_Camera.clipPlanes.y+1.0);
	screenPosition.z=(log2(max(sc_Camera.clipPlanes.x,1.0+screenPosition.w))*fCoefficient -1.0)*screenPosition.w;

	#endif

	gl_Position=screenPosition;
}

#endif

#ifdef FRAGMENT_SHADER

#ifdef RENDER_CONSTANT_COLOR

void main()
{
	gl_FragColor=baseColor;
}

#else

uniform NF_PRECISION float alphaTestThreshold;

void ngsAlphaTest(float opacity)
{
	#if defined(sc_BlendMode_AlphaTest)

	if (opacity < alphaTestThreshold)
	{
		discard;
	}

	#endif

	#if defined(ENABLE_STIPPLE_PATTERN_TEST)

	vec2 localCoord=floor(mod(gl_FragCoord.xy,vec2(4.0)));
	float threshold=(mod(dot(localCoord,vec2(4.0,1.0))*9.0,16.0)+1.0)/17.0;

	if (opacity < threshold)
	{
		discard;
	}

	#endif
}

vec4 ngsPixelShader(vec4 result)
{
	#if defined(sc_ProjectiveShadowsCaster)

	return evaluateShadowCasterColor(result);

	#else

	#if defined(sc_BlendMode_Custom)

	result=applyCustomBlend(result);

	#elif defined(sc_BlendMode_MultiplyOriginal)

	result.rgb=mix(vec3(1.0),result.rgb,result.a);

	#elif defined(sc_BlendMode_Screen)

	result.rgb=result.rgb*result.a;

	#endif

	return result;

	#endif
}

#endif

#endif

#ifdef VERTEX_SHADER

uniform sampler2D aoTex;
uniform mat3 aoTexTransform;
uniform vec4 aoTexUvMinMax;
uniform vec4 aoTexBorderColor;
uniform vec4 aoTexSize;

#ifndef SC_USE_UV_TRANSFORM_aoTex
#define SC_USE_UV_TRANSFORM_aoTex false
#define SC_SOFTWARE_WRAP_MODE_U_aoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_aoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_aoTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_aoTex
#define SC_USE_CLAMP_TO_BORDER_aoTex false
#endif

#ifndef aoTexUV
#define aoTexUV 0
#endif
uniform sampler2D normalOut;
uniform mat3 normalOutTransform;
uniform vec4 normalOutUvMinMax;
uniform vec4 normalOutBorderColor;
uniform vec4 normalOutSize;

#ifndef SC_USE_UV_TRANSFORM_normalOut
#define SC_USE_UV_TRANSFORM_normalOut false
#define SC_SOFTWARE_WRAP_MODE_U_normalOut SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_normalOut SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_normalOut false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_normalOut
#define SC_USE_CLAMP_TO_BORDER_normalOut false
#endif

#ifndef normalOutUV
#define normalOutUV 0
#endif
uniform sampler2D pupilTex;
uniform mat3 pupilTexTransform;
uniform vec4 pupilTexUvMinMax;
uniform vec4 pupilTexBorderColor;
uniform vec4 pupilTexSize;

#ifndef SC_USE_UV_TRANSFORM_pupilTex
#define SC_USE_UV_TRANSFORM_pupilTex false
#define SC_SOFTWARE_WRAP_MODE_U_pupilTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_pupilTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_pupilTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_pupilTex
#define SC_USE_CLAMP_TO_BORDER_pupilTex false
#endif

#ifndef pupilTexUV
#define pupilTexUV 0
#endif
uniform sampler2D normalIn;
uniform mat3 normalInTransform;
uniform vec4 normalInUvMinMax;
uniform vec4 normalInBorderColor;
uniform vec4 normalInSize;

#ifndef SC_USE_UV_TRANSFORM_normalIn
#define SC_USE_UV_TRANSFORM_normalIn false
#define SC_SOFTWARE_WRAP_MODE_U_normalIn SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_normalIn SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_normalIn false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_normalIn
#define SC_USE_CLAMP_TO_BORDER_normalIn false
#endif

#ifndef normalInUV
#define normalInUV 0
#endif
uniform sampler2D scaleMask;
uniform mat3 scaleMaskTransform;
uniform vec4 scaleMaskUvMinMax;
uniform vec4 scaleMaskBorderColor;
uniform vec4 scaleMaskSize;

#ifndef SC_USE_UV_TRANSFORM_scaleMask
#define SC_USE_UV_TRANSFORM_scaleMask false
#define SC_SOFTWARE_WRAP_MODE_U_scaleMask SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_scaleMask SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_scaleMask false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_scaleMask
#define SC_USE_CLAMP_TO_BORDER_scaleMask false
#endif

#ifndef scaleMaskUV
#define scaleMaskUV 0
#endif
uniform NF_PRECISION		float pupilRadius;
uniform sampler2D albedoTex;
uniform mat3 albedoTexTransform;
uniform vec4 albedoTexUvMinMax;
uniform vec4 albedoTexBorderColor;
uniform vec4 albedoTexSize;

#ifndef SC_USE_UV_TRANSFORM_albedoTex
#define SC_USE_UV_TRANSFORM_albedoTex false
#define SC_SOFTWARE_WRAP_MODE_U_albedoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_albedoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_albedoTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_albedoTex
#define SC_USE_CLAMP_TO_BORDER_albedoTex false
#endif

#ifndef albedoTexUV
#define albedoTexUV 0
#endif
uniform NF_PRECISION		float opacity;

struct ssGlobals
{
	bool InVertexShader;
	bool InPixelShader;
};

void main()
{
	ssSetupPreview_Vertex

	sc_Vertex_t v;
	ngsVertexShaderBegin(v);

	ssGlobals Globals;
	Globals.InVertexShader=true;
	Globals.InPixelShader=false;

	float4 ScreenPosition=vec4(0.0);
	float3 WorldPosition=varPos;
	float3 WorldNormal	=varNormal;
	float3 WorldTangent=varTangent;

	if (ssDisableVertexChanges)
	{
		WorldPosition=varPos;
		WorldNormal	=varNormal;
		WorldTangent=varTangent;
	}

	ngsVertexShaderEnd(v,WorldPosition,WorldNormal,WorldTangent,v.position);

	ssPreviewOutput_Vertex
}

#endif

#ifdef FRAGMENT_SHADER

#ifndef UBER_LIGHTING_GLSL
#define UBER_LIGHTING_GLSL

#ifndef PBR_GLSL
#define PBR_GLSL

#ifndef IMPORTANCE_SAMPLING_GLSL
#define IMPORTANCE_SAMPLING_GLSL

#ifndef DEBUG_GLSL
#define DEBUG_GLSL

struct DebugOptions {
	bool enableMetallic;
	bool envBRDFApprox;
	bool envBentNormal;
	float envMip;
	bool envSampling;
	bool envSamplingGroundTruth;
	int envSamples;
	int envRandMod;
	int envRandSeed;
	bool fringelessMetallic;
	bool acesToneMapping;
	bool linearToneMapping;
	bool albedo;
	bool specColor;
	bool roughness;
	bool normal;
	bool ao;
	float directDiffuse;
	float directSpecular;
	float indirectDiffuse;
	float indirectSpecular;
};

DebugOptions defaultDebugOptions(){
	DebugOptions debug;

	debug.enableMetallic=true;
	debug.envBRDFApprox=true;
	debug.envBentNormal=true;
	debug.envMip=-1.0;
	debug.envSampling=false;
	debug.envSamplingGroundTruth=false;
#ifndef MOBILE
	debug.envSamples=500;
	debug.envRandMod=333;
	debug.envRandSeed=-1;
#endif
	debug.fringelessMetallic=true;
	debug.acesToneMapping=false;
	debug.linearToneMapping=false;
	debug.albedo=false;
	debug.specColor=false;
	debug.roughness=false;
	debug.normal=false;
	debug.directDiffuse=1.0;
	debug.indirectDiffuse=1.0;
	debug.directSpecular=1.0;
	debug.indirectSpecular=1.0;

	return debug;
}

#endif

#ifdef ENABLE_IMPORTANCE_SAMPLING

vec3 sampleRadiance(vec3 R);
vec3 fresnelSchlick(float cosTheta,vec3 F0);

float radicalInverse(int n){
	float val=0.0;
	float invBase=0.5;
	float invBi=invBase;
	while (n > 0){
		int d_i=n - ((n/2)*2);
		val+=d_i*invBi;
		n/=2;
		invBi*=0.5;
	}
	return val;
}

vec2 hammersley(int i,int N){
	return vec2(float(i)/float(N),radicalInverse(i));
}

vec3 importanceSampleGGX(vec2 Xi,float roughness,vec3 N){
	float a=roughness*roughness;
	float Phi=2*PI*Xi.x;
	float CosTheta=sqrt((1 -Xi.y)/(1+(a*a -1)*Xi.y));
	float SinTheta=sqrt(1 -CosTheta*CosTheta);
	vec3 H;
	H.x=SinTheta*cos(Phi);
	H.y=SinTheta*sin(Phi);
	H.z=CosTheta;
	vec3 UpVector=abs(N.z) < 0.999 ? vec3(0,0,1) : vec3(1,0,0);
	vec3 TangentX=normalize(cross(UpVector,N));
	vec3 TangentY=cross(N,TangentX);

	return TangentX*H.x+TangentY*H.y+N*H.z;
}

vec3 prefilterEnvmap(float roughness,vec3 R,DebugOptions debug){
	vec3 N=R;
	vec3 V=R;
	vec3 totalColor=vec3(0.0);
	float totalWeight=0.0;

	int randMod=debug.envRandMod;
	int randSeed=debug.envRandSeed;
	if (randSeed < 0){

		randSeed=int((abs(R.x)+abs(R.y)+abs(R.z))*12345711.0);
		randSeed=int(mod(float(randSeed),float(randMod)));
	}

	int numSamples=int(debug.envSamples);
	for(int i=0;i < numSamples;i++){
		vec2 Xi=hammersley(i*randMod+randSeed,numSamples*randMod);
		vec3 H=importanceSampleGGX(Xi,roughness,N);
		vec3 L=2*dot(V,H)*H -V;

		float NoL=saturate(dot(N,L));

		if(NoL > 0){
			vec3 IncidentLight=sampleRadiance(L)*NoL;
			totalColor+=IncidentLight;
			totalWeight+=NoL;
		}
	}
	return totalColor/totalWeight;
}

float gEnv1(vec3 N,vec3 V,float roughness){

	float alpha=roughness*roughness;
	float k=alpha*0.5;
	float NdotV=saturate(dot(N,V));
	return NdotV*(1.0 -k)+k;
}

float gEnv(vec3 N,vec3 L,vec3 V,float roughness){
	return 1.0/(gEnv1(N,L,roughness)*gEnv1(N,V,roughness));
}

vec3 calculateIndirectSpecularGroundTruth(float roughness,vec3 specColor,vec3 N,vec3 V,DebugOptions debug){
#ifdef MAYA
	N=vec3(-N.z,N.y,-N.x);
	V=vec3(-V.z,V.y,-V.x);
#endif

	vec3 R=reflect(-V,N);
	int randMod=int(debug.envRandMod);
	int rand=int((abs(R.x)+abs(R.y)+abs(R.z))*12345711.0);
	rand=int(mod(float(rand),float(randMod)));

	vec3 totalColor=vec3(0.0);
	int numSamples=int(debug.envSamples);
	for(int i=0;i < numSamples;i++){
		vec2 Xi=hammersley(i*randMod+rand,numSamples*randMod);
		vec3 H=importanceSampleGGX(Xi,roughness,N);
		vec3 L=2*dot(V,H)*H -V;

		float NoL=saturate(dot(N,L));
		float NoH=saturate(dot(N,H));
		float VoH=saturate(dot(V,H));

		if(NoL > 0){
			vec3 IncidentLight=sampleRadiance(L)*NoL;

			totalColor+=IncidentLight*gEnv(N,L,V,roughness)*VoH*(1.0/NoH)*fresnelSchlick(VoH,specColor);
		}
	}
	return totalColor/numSamples;
}

#endif

#endif

#if defined(FRAGMENT_SHADER) || !defined(SCENARIUM)

vec3 sampleSpecularEnvmapLod(vec3 R,float lod);
vec3 sampleRadiance(vec3 R);
vec3 calculateDiffuseIrradiance(vec3 N);

struct SurfaceProperties {
	vec3 albedo;
	float opacity;
	vec3 normal;
	float metallic;
	float roughness;
	vec3 emissive;
	vec3 ao;
	vec3 specularAo;
	vec3 bakedShadows;

	vec3 specColor;
};

struct LightingComponents {
	vec3 directDiffuse;
	vec3 directSpecular;
	vec3 indirectDiffuse;
	vec3 indirectSpecular;
	vec3 emitted;
	vec3 transmitted;
};

struct LightProperties {
	vec3 direction;
	vec3 color;
	float attenuation;
};

SurfaceProperties defaultSurfaceProperties(){
	SurfaceProperties surfaceProperties;

	surfaceProperties.albedo=vec3(0.0);
	surfaceProperties.opacity=1.0;
	surfaceProperties.normal=vec3(0.0);
	surfaceProperties.metallic=0.0;
	surfaceProperties.roughness=0.0;
	surfaceProperties.emissive=vec3(0.0);
	surfaceProperties.ao=vec3(1.0);
	surfaceProperties.specularAo=vec3(1.0);
	surfaceProperties.bakedShadows=vec3(1.0);

	return surfaceProperties;
}

LightingComponents defaultLightingComponents(){
	LightingComponents lighting;

	lighting.directDiffuse=vec3(0.0);
	lighting.directSpecular=vec3(0.0);
	lighting.indirectDiffuse=vec3(1.0);
	lighting.indirectSpecular=vec3(0.0);
	lighting.emitted=vec3(0.0);
	lighting.transmitted=vec3(0.0);

	return lighting;
}

#if defined(ENABLE_LIGHTING) || !defined(SCENARIUM)

vec3 fresnelSchlickSub(float cosTheta,vec3 F0,vec3 fresnelMax){
	float b=1.0 -cosTheta;
	float b2=b*b;
	float b5=b2*b2*b;
	return F0+(fresnelMax -F0)*b5;
}

vec3 fresnelSchlick(float cosTheta,vec3 F0){
	return fresnelSchlickSub(cosTheta,F0,vec3(1.0));
}

float Dggx(float NdotH,float roughness){
	float a=roughness*roughness;
	float a2=a*a;
	float NdotH2=NdotH*NdotH;
	float b=NdotH2*(a2 -1.0)+1.0;
	float b2=b*b;
	return a2/b2;
}

float Gggx1(float NdotV,float roughness){
	float k=roughness+1.0;
	k=k*k*0.125;
	return NdotV*(1.0 -k)+k;
}

float Gggx(float NdotL,float NdotV,float roughness){

	return 1.0/(Gggx1(NdotL,roughness)*Gggx1(NdotV,roughness));
}

vec3 calculateDirectDiffuse(SurfaceProperties surfaceProperties,vec3 L){
	return vec3(saturate(dot(surfaceProperties.normal,L)));
}

vec3 calculateDirectSpecular(SurfaceProperties surfaceProperties,vec3 L,vec3 V){
	float r=max(surfaceProperties.roughness,0.03);
	vec3 F0=surfaceProperties.specColor;
	vec3 N=surfaceProperties.normal;
	vec3 H=normalize(L+V);
	float NdotL=saturate(dot(N,L));
	float NdotV=saturate(dot(N,V));
	float NdotH=saturate(dot(N,H));
	float VdotH=saturate(dot(V,H));

#if defined(DEVICE_IS_FAST)

	return Dggx(NdotH,r)*Gggx(NdotL,NdotV,r)*0.25*NdotL*fresnelSchlick(VdotH,F0);
#else

	float specPower=exp2(11.0 -10.0*r);
	return fresnelSchlick(VdotH,F0)*(specPower*0.125+0.25)*pow(NdotH,specPower)*NdotL;
#endif
}

LightingComponents accumulateLight(LightingComponents lighting,LightProperties light,SurfaceProperties surfaceProperties,vec3 V){
	lighting.directDiffuse+=calculateDirectDiffuse(surfaceProperties,light.direction)*light.color*light.attenuation;
	lighting.directSpecular+=calculateDirectSpecular(surfaceProperties,light.direction,V)*light.color*light.attenuation;
	return lighting;
}

#ifndef SUBSTANCE

vec3 calculateIndirectDiffuse(SurfaceProperties surfaceProperties){
	return calculateDiffuseIrradiance(surfaceProperties.normal);
}

#ifdef SCENARIUM
const float sc_RoughnessExponentInv=1.0/1.5;
const float sc_MaxRoughnessMip=5.0;
#endif

float calculateEnvMipFromRoughness(float roughness,DebugOptions debug){
#ifdef SCENARIUM

	float mip=saturate(pow(roughness,sc_RoughnessExponentInv))*sc_MaxRoughnessMip;
#else

	float gloss=1.0 -roughness;
	float lysRoughness=1.0 -gloss*gloss;
	const float maxRoughnessMip=7.0;
	float mip=lysRoughness*maxRoughnessMip;
#endif

#ifdef DEBUG
	if (debug.envMip >=0.0){
		mip=debug.envMip;
	}
#endif

	return mip;
}

vec3 getSpecularDominantDir(vec3 N,vec3 R,float roughness,DebugOptions debug){
#ifdef DEVICE_IS_FAST
#ifdef DEBUG
	if (debug.envBentNormal)
#endif
	{
		float lerpFactor=roughness*roughness*roughness;
		return normalize(mix(R,N,lerpFactor));
	}
#else
	return R;
#endif
}

vec3 envBRDFApprox(SurfaceProperties surfaceProperties,float NdotV,DebugOptions debug){
#ifdef DEVICE_IS_FAST
#ifdef DEBUG
	bool useBRDFApprox=debug.envBRDFApprox;
#else
	bool useBRDFApprox=true;
#endif
#else
	bool useBRDFApprox=false;
#endif

	if (useBRDFApprox){
		const vec4 c0=vec4(-1.0,-0.0275,-0.572,0.022);
		const vec4 c1=vec4(1.0,0.0425,1.04,-0.04);
		vec4 r=surfaceProperties.roughness*c0+c1;
		float a004=min(r.x*r.x,exp2(-9.28*NdotV))*r.x+r.y;
		vec2 AB=vec2(-1.04,1.04)*a004+r.zw;
		return max(surfaceProperties.specColor*AB.x+AB.y,vec3(0.0));
	}
	else {

		vec3 fresnelMax=max(vec3(1.0 -surfaceProperties.roughness),surfaceProperties.specColor);
		return fresnelSchlickSub(NdotV,surfaceProperties.specColor,fresnelMax);
	}
}

vec3 calculateIndirectSpecular(SurfaceProperties surfaceProperties,vec3 V,DebugOptions debug){
#ifdef MAYA
	if (debug.envSamplingGroundTruth){
		return calculateIndirectSpecularGroundTruth(surfaceProperties,V,debug);
	}
#endif

	vec3 N=surfaceProperties.normal;

#ifdef MAYA

	N=vec3(-N.z,N.y,-N.x);
	V=vec3(-V.z,V.y,-V.x);
#endif

	vec3 R=reflect(-V,N);

	R=getSpecularDominantDir(N,R,surfaceProperties.roughness,debug);

	float mip=calculateEnvMipFromRoughness(surfaceProperties.roughness,debug);

	vec3 envmap=sampleSpecularEnvmapLod(R,mip);

#ifdef MAYA
	if (debug.envSampling){
		envmap=prefilterEnvmap(surfaceProperties.roughness,R,debug);
	}
#endif

	return envmap*envBRDFApprox(surfaceProperties,abs(dot(N,V)),debug);
}
#endif

void deriveAlbedoAndSpecColorFromSurfaceProperties(in SurfaceProperties surfaceProperties,out vec3 albedo,out vec3 specColor,DebugOptions debug){
#ifdef DEBUG
	if (debug.enableMetallic){
		if (debug.fringelessMetallic){
#endif

			specColor=mix(vec3(0.04),surfaceProperties.albedo*surfaceProperties.metallic,surfaceProperties.metallic);
			albedo=mix(surfaceProperties.albedo*(1.0-surfaceProperties.metallic),vec3(0.0),surfaceProperties.metallic);
#ifdef DEBUG
		} else {
			specColor=mix(vec3(0.04),surfaceProperties.albedo,surfaceProperties.metallic);
			albedo=mix(surfaceProperties.albedo,vec3(0.0),surfaceProperties.metallic);
		}
	} else {

		specColor=surfaceProperties.specColor;
		albedo=surfaceProperties.albedo;
	}
#endif
}

SurfaceProperties calculateDerivedSurfaceProperties(SurfaceProperties surfaceProperties,DebugOptions debug){
	deriveAlbedoAndSpecColorFromSurfaceProperties(surfaceProperties,surfaceProperties.albedo,surfaceProperties.specColor,debug);
	return surfaceProperties;
}

#endif

vec3 combineSurfacePropertiesWithLighting(SurfaceProperties surfaceProperties,LightingComponents lighting,bool enablePremultipliedAlpha){
	vec3 diffuse=surfaceProperties.albedo*(lighting.directDiffuse+lighting.indirectDiffuse*surfaceProperties.ao);
	vec3 specular=lighting.directSpecular+lighting.indirectSpecular*surfaceProperties.specularAo;
	vec3 emitted=surfaceProperties.emissive;
	vec3 transmitted=lighting.transmitted;

	if (enablePremultipliedAlpha){
		diffuse*=srgbToLinear(surfaceProperties.opacity);
	}

	vec3 result=diffuse+specular+emitted+transmitted;

	return result;
}

vec4 debugOutput(vec4 regularOutput,SurfaceProperties surfaceProperties,LightingComponents lighting,DebugOptions debug){
	vec4 result=regularOutput;
	result.a=1.0;

	if (debug.albedo){
		result.xyz=surfaceProperties.albedo;
	} else if (debug.specColor){
		result.xyz=surfaceProperties.specColor;
	} else if (debug.roughness){
		result.xyz=vec3(srgbToLinear(surfaceProperties.roughness));
	} else if (debug.normal){
		result.xyz=srgbToLinear(surfaceProperties.normal*0.5+0.5);
	} else if (debug.ao){
		result.xyz=surfaceProperties.ao;
	} else {
		result=regularOutput;
	}

	return result;
}

vec3 linearToneMapping(vec3 x){

	float a=1.8;
	float b=1.4;
	float c=0.5;
	float d=1.5;

#ifdef GL_FRAGMENT_PRECISION_HIGH
	return (x*(a*x+b))/(x*(a*x+c)+d);
#else
	return (x*(a*x+b))/min(vec3(1000.0),(x*(a*x+c)+d));
#endif
}

vec3 acesToneMapping(vec3 x){

	float a=2.51;
	float b=0.03;
	float c=2.43;
	float d=0.59;
	float e=0.14;

#ifdef GL_FRAGMENT_PRECISION_HIGH
	return (x*(a*x+b))/(x*(c*x+d)+e);
#else
	return (x*(a*x+b))/min(vec3(1000.0),(x*(c*x+d)+e));
#endif
}

#endif
#endif

#ifndef ENVMAP_GLSL
#define ENVMAP_GLSL

#if defined(FRAGMENT_SHADER) || !defined(SCENARIUM)
#if defined(ENABLE_LIGHTING) || !defined(SCENARIUM)

vec4 encodeRGBD(vec3 rgb){
	float maxRGB=max(1.0,(max(rgb.x,max(rgb.g,rgb.b))));
	float D=1.0/maxRGB;
	return vec4(rgb.rgb*D,D);
}

vec3 decodeRGBD(vec4 rgbd){
	return rgbd.rgb*(1.0/rgbd.a);
}

vec3 decodeRGBE(vec4 rgbe){
	float f1=exp2(rgbe.w*255.0 -128.0);
	return rgbe.xyz*vec3(f1);
}

vec2 calcSeamlessPanoramicUvsForConvolution(vec2 uv,vec2 topMipRes,float lod){

	vec2 thisMipRes=max(vec2(1.0),topMipRes/vec2(exp2(lod)));
	return (uv*thisMipRes -0.5)/(thisMipRes -1.0);
}

vec2 calcSeamlessPanoramicUvsForSampling(vec2 uv,vec2 topMipRes,float lod){
#ifdef DEVICE_IS_FAST

	vec2 thisMipRes=max(vec2(1.0),topMipRes/vec2(exp2(lod)));
	return uv*(thisMipRes -1.0)/thisMipRes+0.5/thisMipRes;
#else
	return uv;
#endif
}

float _atan2(float x,float y){

	float signx=x < 0.0 ? -1.0 : 1.0;
	return signx*acos(clamp(y/length(vec2(x,y)),-1.0,1.0));
}

vec2 calcPanoramicTexCoordsFromDir(vec3 reflDir,float rotationDegrees){
	vec2 uv;
	uv.x=_atan2(reflDir.x,-reflDir.z) -PI/2.0;
	uv.y=acos(reflDir.y);
	uv=uv/vec2(2.0*PI,PI);
	uv.y=1.0 -uv.y;

	uv.x+=rotationDegrees/360.0;
	uv.x=fract(uv.x+floor(uv.x)+1.0);

	return uv;
}

vec3 calcDirFromPanoramicTexCoords(vec2 uv){
	float a=2.0*PI*(uv.x);
	float b=PI*uv.y;

	float x=sin(a)*sin(b);
	float y=cos(b);
	float z=cos(a)*sin(b);

	return vec3(z,y,x);
}

vec2 calcAngularTexCoordsFromDir(vec3 V){
	V=vec3(-V.z,V.y,-V.x);
	float r=0.159154943*acos(V.z)/sqrt(V.x*V.x+V.y*V.y);
	float u=0.5+V.x*r;
	float v=0.5+V.y*r;
	return vec2(u,1.0 -v);
}

vec2 calculateEnvmapScreenToCube(vec3 V)
{

	V.z=abs(V.z);
	vec3 vAbs=abs(V);
	vec2 uv;
	if(vAbs.z >=vAbs.x && vAbs.z >=vAbs.y)
	{
		float ma=0.5/vAbs.z;
		uv=vec2(V.x,V.y)*ma;
		uv=uv*0.5+vec2(0.5);
	}
	else if(vAbs.y >=vAbs.x)
	{
		float ma=0.5/vAbs.y;
		uv=vec2(V.x,-V.z)*ma;
		uv.x=uv.x*mix(0.5,1.0,1.0 -abs(uv.y)*2.0);
		uv.x+=0.5;
		uv.y*=0.5;
		uv.y=abs(uv.y);
		if (V.y > 0.0){
			uv.y=1.0 -uv.y;
		}
	}
	else
	{
		float ma=0.5/vAbs.x;
		uv=vec2(V.x < 0.0 ? V.z : -V.z,V.y)*ma;
		uv.y=uv.y*mix(0.5,1.0,1.0 -abs(uv.x)*2.0);
		uv.y+=0.5;
		uv.x*=0.5;
		uv.x=abs(uv.x);
		if (V.x > 0.0){
			uv.x=1.0 -uv.x;
		}
	}
	return uv;
}

#endif

vec2 calcSphericalTexCoordsFromDir(vec3 reflDir){
	float m=2.0*sqrt(reflDir.x*reflDir.x+reflDir.y*reflDir.y+(reflDir.z+1.0)*(reflDir.z+1.0));
	vec2 reflTexCoord=reflDir.xy/m+0.5;
	return reflTexCoord;
}

#endif
#endif

#ifndef SPHERICAL_HARMONICS_GLSL
#define SPHERICAL_HARMONICS_GLSL

#ifdef ENABLE_SH_INTEGRATION

vec3 sampleRadiance(vec3 R);

const int NumShCoeffs=9;

void updateShCoeffs(inout vec3 coeffs[NumShCoeffs],vec3 hdr,float domega,float x,float y,float z){

	int col;
	for (col=0;col < 3;col++){
		float c;

		c=0.282095;
		coeffs[0][col]+=hdr[col]*c*domega;

		c=0.488603;
		coeffs[1][col]+=hdr[col]*(c*y)*domega;
		coeffs[2][col]+=hdr[col]*(c*z)*domega;
		coeffs[3][col]+=hdr[col]*(c*x)*domega;

		c=1.092548;
		coeffs[4][col]+=hdr[col]*(c*x*y)*domega;
		coeffs[5][col]+=hdr[col]*(c*y*z)*domega;
		coeffs[7][col]+=hdr[col]*(c*x*z)*domega;

		c=0.315392;
		coeffs[6][col]+=hdr[col]*(c*(3.0*z*z-1.0))*domega;

		c=0.546274;
		coeffs[8][col]+=hdr[col]*(c*(x*x-y*y))*domega;
	}
}

float sinc(float x){
	if (abs(x) < 1.0e-4) return 1.0;
	else return(sin(x)/x);
}

vec3 projectEnvmapToShCoefficient(int coeffIndex,int width){
	vec3 coeffs[NumShCoeffs];
	for (int i=0;i < NumShCoeffs;++i){
		coeffs[i]=vec3(0,0,0);
	}

	for (int i=0;i < width;i++){
		for (int j=0;j < width;j++){

			float u,v,r,theta,phi,x,y,z,domega;

			v=(width/2.0 -i)/(width/2.0);
			u=(j-width/2.0)/(width/2.0);
			r=sqrt(u*u+v*v);
			if (r > 1.0) continue;

			theta=PI*r;
			phi=atan(v,u);

			x=sin(theta)*cos(phi);
			y=sin(theta)*sin(phi);
			z=cos(theta);

			domega=(2.0*PI/width)*(2.0*PI/width)*sinc(theta)/PI;

			vec3 hdr=sampleRadiance(vec3(x,y,z));

			updateShCoeffs(coeffs,hdr,domega,x,y,z);
		}
	}

	return coeffs[coeffIndex];
}

#endif

vec3 evaluateSh(vec3 L00,vec3 L1_1,vec3 L10,vec3 L11,vec3 L2_2,vec3 L2_1,vec3 L20,vec3 L21,vec3 L22,vec3 n){
	float x,y,z,x2,y2,z2,xy,yz,xz;

	const float c1=0.429043;
	const float c2=0.511664;
	const float c3=0.743125;
	const float c4=0.886227;
	const float c5=0.247708;
	x=n[0];y=n[1];z=n[2];

	x2=x*x;y2=y*y;z2=z*z;
	xy=x*y;yz=y*z;xz=x*z;

	return c1*L22*(x2-y2)+c3*L20*z2+c4*L00 -c5*L20
	+2.0*c1*(L2_2*xy+L21*xz+L2_1*yz)
	+2.0*c2*(L11*x+L1_1*y+L10*z);
}

#endif

#ifdef FRAGMENT_SHADER
#ifdef ENABLE_LIGHTING

#define MAX_MIP_BIAS 13.0

uniform float envmapExposure;
uniform float envmapRotation;
uniform float reflBlurWidth;
uniform float reflBlurMinRough;
uniform float reflBlurMaxRough;

vec3 sampleDiffuseEnvmap(vec3 N){
	vec2 uv=calcPanoramicTexCoordsFromDir(N,sc_EnvmapRotation.y);

#if (sc_MaxTextureImageUnits > 8) && defined(sc_HasDiffuseEnvmap)

	uv=calcSeamlessPanoramicUvsForSampling(uv,sc_EnvmapDiffuseSize.xy,0.0);
	vec4 tex=texture2D(sc_EnvmapDiffuse,uv,-MAX_MIP_BIAS);

#else

	vec4 tex=sc_texture2DLod(sc_EnvmapSpecular,sc_EnvmapSpecularSize.xy,uv,sc_MaxRoughnessMip);
#endif

	return decodeRGBD(tex)*sc_EnvmapExposure;
}

vec3 calculateDiffuseIrradiance(vec3 N){
	vec3 accumulatedLight=vec3(0.0);

#ifdef sc_EnvLightMode
#if sc_EnvLightMode==sc_AmbientLightMode_SphericalHarmonics
	accumulatedLight=evaluateSh(sc_Sh[0],sc_Sh[1],sc_Sh[2],sc_Sh[3],sc_Sh[4],sc_Sh[5],sc_Sh[6],sc_Sh[7],sc_Sh[8],-N)*sc_ShIntensity;
#elif sc_EnvLightMode==sc_AmbientLightMode_EnvironmentMap
	accumulatedLight=sampleDiffuseEnvmap(N);
#endif
#endif

#ifdef sc_AmbientLightsCount
#if sc_AmbientLightsCount > 0
#if sc_AmbientLightMode0==sc_AmbientLightMode_Constant
	accumulatedLight+=sc_AmbientLights[0].color*sc_AmbientLights[0].intensity;
#else
	accumulatedLight.r+=SC_EPSILON*sc_AmbientLights[0].color.r;
#endif
#endif
#if sc_AmbientLightsCount > 1
#if sc_AmbientLightMode1==sc_AmbientLightMode_Constant
	accumulatedLight+=sc_AmbientLights[1].color*sc_AmbientLights[1].intensity;
#else
	accumulatedLight.r+=SC_EPSILON*sc_AmbientLights[1].color.r;
#endif
#endif
#if sc_AmbientLightsCount > 2
#if sc_AmbientLightMode2==sc_AmbientLightMode_Constant
	accumulatedLight+=sc_AmbientLights[2].color*sc_AmbientLights[2].intensity;
#else
	accumulatedLight.r+=SC_EPSILON*sc_AmbientLights[2].color.r;
#endif
#endif
#endif

	return accumulatedLight;
}

vec4 emulateTexture2DLod(sampler2D sampler,vec2 textureSize,vec2 uv,float lod){
#if (__VERSION__==120)
	return texture2DLod(sampler,uv,lod);
#elif defined(GL_EXT_shader_texture_lod)
	return texture2DLodEXT(sampler,uv,lod);
#elif defined(GL_OES_standard_derivatives)
	vec2 texels=uv*textureSize;
	float dudx=dFdx(texels.x);
	float dvdx=dFdx(texels.y);
	float dudy=dFdy(texels.x);
	float dvdy=dFdy(texels.y);
	float rho=max(length(vec2(dudx,dvdx)),length(vec2(dudy,dvdy)));
	float mu=max(abs(dudx),abs(dudy));
	float mv=max(abs(dvdx),abs(dvdy));
	float rho2=max(mu,mv);
	float mip=log2(rho2);
	float bias=lod -mip;
	return texture2D(sampler,uv,bias);
#else
	#error Unsupported call to emulateTexture2DLod
#endif
}

vec4 sampleSpecularEnvTextureLod(vec2 uv,float lod){
#if (__VERSION__==120) || defined(GL_EXT_shader_texture_lod) || defined(GL_OES_standard_derivatives)
	return emulateTexture2DLod(sc_EnvmapSpecular,sc_EnvmapSpecularSize.xy,uv,lod);
#else

	vec4 spec=texture2D(sc_EnvmapSpecular,uv,-MAX_MIP_BIAS);
	vec4 diff=texture2D(sc_EnvmapSpecular,uv,sc_MaxRoughnessMip);
	return mix(spec,diff,lod/sc_MaxRoughnessMip);
#endif
}

vec3 sampleSpecularEnvTextureLod(vec3 R,float lod){
	vec2 uv=calcPanoramicTexCoordsFromDir(R,sc_EnvmapRotation.y);

#if defined(DEVICE_IS_FAST)

	float lodFloor=floor(lod);
	float lodCeil=ceil(lod);
	float lodFrac=lod -lodFloor;

	vec2 uvFloor=calcSeamlessPanoramicUvsForSampling(uv,sc_EnvmapSpecularSize.xy,lodFloor);
	vec4 texFloor=sampleSpecularEnvTextureLod(uvFloor,lodFloor);

	vec2 uvCeil=calcSeamlessPanoramicUvsForSampling(uv,sc_EnvmapSpecularSize.xy,lodCeil);
	vec4 texCeil=sampleSpecularEnvTextureLod(uvCeil,lodCeil);

	vec4 tex=mix(texFloor,texCeil,lodFrac);
#else
	vec4 tex=sampleSpecularEnvTextureLod(uv,lod);
#endif

	return decodeRGBD(tex);
}

vec3 sampleScreenTexture(vec2 uv,float lod)
{
#if defined(ENABLE_ENVMAP_FROM_CAMERA_ROUGHNESS) && defined(DEVICE_IS_FAST)
	const float maxRoughnessMipInv=1.0/5.0;
	float r=lod*maxRoughnessMipInv;
	float reflectionRoughness=saturate((r -reflBlurMinRough)/(reflBlurMaxRough-reflBlurMinRough));
	vec2 pixelSize=vec2(1.0/720.0,1.0/1280.0);
	vec3 blurred=vec3(0.0);
	const int NUM_SAMPLES=5;
	vec2 offset=pixelSize*reflBlurWidth/float(NUM_SAMPLES)*reflectionRoughness;
	vec2 rnd=fract(uv*1331711.0) -0.5;

	uv -=offset*float(NUM_SAMPLES-1)*0.5;
	for (int i=0;i < NUM_SAMPLES;++i){
		for (int j=0;j < NUM_SAMPLES;++j){
			blurred+=texture2D(sc_ScreenTexture,uv+offset*vec2(i,j)).rgb;
		}
	}
	blurred*=1.0/float(NUM_SAMPLES*NUM_SAMPLES);
	return blurred;
#else
	return texture2D(sc_ScreenTexture,uv).rgb;
#endif
}

vec3 sampleSpecularEnvmapLod(vec3 R,float lod){
#ifdef ENABLE_ENVMAP_FROM_CAMERA

	R=(sc_ViewMatrix*vec4(R,0.0)).xyz;
	vec2 uv=calculateEnvmapScreenToCube(R);
	return srgbToLinear(sampleScreenTexture(uv,lod));

#else

	vec3 res=sampleSpecularEnvTextureLod(R,lod)*sc_EnvmapExposure;

#ifndef MOBILE
	res+=vec3(SC_EPSILON);
#endif

	return res;

#endif
}

LightingComponents evaluateLighting(SurfaceProperties surfaceProperties,DebugOptions debug){
	LightingComponents lighting=defaultLightingComponents();

	vec3 V=normalize(sc_Camera.position -varPos);

	int numLights=0;
	vec4 bakedShadows=vec4(surfaceProperties.bakedShadows,1.0);

#ifdef sc_DirectionalLightsCount

	for(int i=0;i < sc_DirectionalLightsCount;++i){
		sc_DirectionalLight_t light=sc_DirectionalLights[i];
		LightProperties lightProperties;
		lightProperties.direction=light.direction;
		lightProperties.color=light.color.rgb;
		lightProperties.attenuation=light.color.a;
		lightProperties.attenuation*=bakedShadows[numLights < 3 ? numLights : 3];
		numLights++;
		lighting=accumulateLight(lighting,lightProperties,surfaceProperties,V);
	}
#endif

#ifdef sc_PointLightsCount

	for(int i=0;i < sc_PointLightsCount;++i){
		sc_PointLight_t light=sc_PointLights[i];
		LightProperties lightProperties;
		lightProperties.direction=normalize(light.position -varPos);
		lightProperties.color=light.color.rgb;
		lightProperties.attenuation=light.color.a;
		lightProperties.attenuation*=bakedShadows[numLights < 3 ? numLights : 3];
		numLights++;
		lighting=accumulateLight(lighting,lightProperties,surfaceProperties,V);
	}
#endif

#ifdef sc_ProjectiveShadowsReceiver
	vec3 shadowColor=evaluateShadow();
	lighting.directDiffuse*=shadowColor;
	lighting.directSpecular*=shadowColor;
#endif

#ifndef ENABLE_SPECULAR_LIGHTING
	lighting.directSpecular=vec3(0.0);
#endif

#ifndef ENABLE_DIFFUSE_LIGHTING
	lighting.directDiffuse=vec3(0.0);
#endif

#ifdef ENABLE_DIFFUSE_LIGHTING
	lighting.indirectDiffuse=calculateIndirectDiffuse(surfaceProperties);
#else
	lighting.indirectDiffuse=vec3(0.0);
#endif

#ifdef sc_EnvLightMode
#if sc_EnvLightMode==sc_AmbientLightMode_EnvironmentMap
	#define ENVMAP_IS_ENABLED
#endif
#endif
#if defined(ENABLE_SPECULAR_LIGHTING) && !defined(ENABLE_SIMPLE_REFLECTION) && (defined(ENVMAP_IS_ENABLED) || defined(ENABLE_ENVMAP_FROM_CAMERA))
	lighting.indirectSpecular=calculateIndirectSpecular(surfaceProperties,V,debug)+vec3(0.001);
#endif

#ifdef ENABLE_TRANSLUCENCY_BROAD
#endif

#ifdef DEBUG

	lighting.directDiffuse*=debug.directDiffuse;
	lighting.directSpecular*=debug.directSpecular;
	lighting.indirectDiffuse*=debug.indirectDiffuse;
	lighting.indirectSpecular*=debug.indirectSpecular;
#endif

	return lighting;
}

#endif
#endif

#endif

#ifndef UBER_DEBUG_GLSL
#define UBER_DEBUG_GLSL

#ifdef FRAGMENT_SHADER

#ifdef DEBUG
uniform int DebugAlbedo;
uniform int DebugSpecColor;
uniform int DebugRoughness;
uniform int DebugNormal;
uniform int DebugAo;
uniform float DebugDirectDiffuse;
uniform float DebugDirectSpecular;
uniform float DebugIndirectDiffuse;
uniform float DebugIndirectSpecular;
uniform float DebugRoughnessOffset;
uniform float DebugRoughnessScale;
uniform float DebugNormalIntensity;
uniform int DebugEnvBRDFApprox;
uniform int DebugEnvBentNormal;
uniform float DebugEnvMip;
uniform int DebugFringelessMetallic;
uniform int DebugAcesToneMapping;
uniform int DebugLinearToneMapping;
#endif

DebugOptions setupDebugOptions(){
	DebugOptions debug=defaultDebugOptions();

#ifdef DEBUG
	debug.envMip=DebugEnvMip;
	debug.envBRDFApprox=bool(DebugEnvBRDFApprox);
	debug.envBentNormal=bool(DebugEnvBentNormal);
	debug.fringelessMetallic=bool(DebugFringelessMetallic);
	debug.acesToneMapping=bool(DebugAcesToneMapping);
	debug.linearToneMapping=bool(DebugLinearToneMapping);
	debug.albedo=bool(DebugAlbedo);
	debug.specColor=bool(DebugSpecColor);
	debug.roughness=bool(DebugRoughness);
	debug.normal=bool(DebugNormal);
	debug.ao=bool(DebugAo);
	debug.directDiffuse=DebugDirectDiffuse;
	debug.directSpecular=DebugDirectSpecular;
	debug.indirectDiffuse=DebugIndirectDiffuse;
	debug.indirectSpecular=DebugIndirectSpecular;
#endif

	return debug;
}

#endif

#endif

vec4 ngsCalculateLighting(vec3 albedo,float opacity,vec3 normal,vec3 emissive,float metallic,float roughness,vec3 ao,vec3 specularAO)
{
	SurfaceProperties surfaceProperties=defaultSurfaceProperties();
	surfaceProperties.opacity=opacity;
	surfaceProperties.albedo=srgbToLinear(albedo);
	surfaceProperties.normal=normalize(normal);
	surfaceProperties.emissive=srgbToLinear(emissive);
	surfaceProperties.metallic=metallic;
	surfaceProperties.roughness=roughness;
	surfaceProperties.ao=ao;
	surfaceProperties.specularAo=specularAO;

	DebugOptions debug=setupDebugOptions();

	#ifdef ENABLE_LIGHTING

	surfaceProperties=calculateDerivedSurfaceProperties(surfaceProperties,debug);
	LightingComponents lighting=evaluateLighting(surfaceProperties,debug);

	#else

	LightingComponents lighting=defaultLightingComponents();

	#endif

	#if defined(sc_BlendMode_ColoredGlass)

	lighting.directDiffuse=vec3(0.0);
	lighting.indirectDiffuse=vec3(0.0);
	vec3 framebuffer=srgbToLinear(getFramebufferColor().rgb);
	lighting.transmitted=framebuffer*mix(vec3(1.0),surfaceProperties.albedo,surfaceProperties.opacity);
	surfaceProperties.opacity=1.0;

	#endif

	#if defined(sc_BlendMode_PremultipliedAlpha)

	const bool enablePremultipliedAlpha=true;

	#else

	const bool enablePremultipliedAlpha=false;

	#endif

	vec4 Output=vec4(combineSurfacePropertiesWithLighting(surfaceProperties,lighting,enablePremultipliedAlpha),surfaceProperties.opacity);

	#if !defined(sc_BlendMode_Multiply)

	#if defined(ENABLE_TONE_MAPPING)

	#if defined(DEBUG)

	if (debug.acesToneMapping)
	Output.rgb=acesToneMapping(Output.rgb);
	else if (debug.linearToneMapping)

	#endif

	Output.rgb=linearToneMapping(Output.rgb);

	#endif

	#endif

	Output.rgb=linearToSrgb(Output.rgb);

	#if defined(DEBUG)

	Output=debugOutput(Output,surfaceProperties,lighting,debug);

	#endif

	return Output;
}

uniform sampler2D aoTex;
uniform mat3 aoTexTransform;
uniform vec4 aoTexUvMinMax;
uniform vec4 aoTexBorderColor;
uniform vec4 aoTexSize;

#ifndef SC_USE_UV_TRANSFORM_aoTex
#define SC_USE_UV_TRANSFORM_aoTex false
#define SC_SOFTWARE_WRAP_MODE_U_aoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_aoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_aoTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_aoTex
#define SC_USE_CLAMP_TO_BORDER_aoTex false
#endif

#ifndef aoTexUV
#define aoTexUV 0
#endif
uniform sampler2D normalOut;
uniform mat3 normalOutTransform;
uniform vec4 normalOutUvMinMax;
uniform vec4 normalOutBorderColor;
uniform vec4 normalOutSize;

#ifndef SC_USE_UV_TRANSFORM_normalOut
#define SC_USE_UV_TRANSFORM_normalOut false
#define SC_SOFTWARE_WRAP_MODE_U_normalOut SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_normalOut SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_normalOut false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_normalOut
#define SC_USE_CLAMP_TO_BORDER_normalOut false
#endif

#ifndef normalOutUV
#define normalOutUV 0
#endif
uniform sampler2D pupilTex;
uniform mat3 pupilTexTransform;
uniform vec4 pupilTexUvMinMax;
uniform vec4 pupilTexBorderColor;
uniform vec4 pupilTexSize;

#ifndef SC_USE_UV_TRANSFORM_pupilTex
#define SC_USE_UV_TRANSFORM_pupilTex false
#define SC_SOFTWARE_WRAP_MODE_U_pupilTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_pupilTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_pupilTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_pupilTex
#define SC_USE_CLAMP_TO_BORDER_pupilTex false
#endif

#ifndef pupilTexUV
#define pupilTexUV 0
#endif
uniform sampler2D normalIn;
uniform mat3 normalInTransform;
uniform vec4 normalInUvMinMax;
uniform vec4 normalInBorderColor;
uniform vec4 normalInSize;

#ifndef SC_USE_UV_TRANSFORM_normalIn
#define SC_USE_UV_TRANSFORM_normalIn false
#define SC_SOFTWARE_WRAP_MODE_U_normalIn SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_normalIn SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_normalIn false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_normalIn
#define SC_USE_CLAMP_TO_BORDER_normalIn false
#endif

#ifndef normalInUV
#define normalInUV 0
#endif
uniform sampler2D scaleMask;
uniform mat3 scaleMaskTransform;
uniform vec4 scaleMaskUvMinMax;
uniform vec4 scaleMaskBorderColor;
uniform vec4 scaleMaskSize;

#ifndef SC_USE_UV_TRANSFORM_scaleMask
#define SC_USE_UV_TRANSFORM_scaleMask false
#define SC_SOFTWARE_WRAP_MODE_U_scaleMask SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_scaleMask SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_scaleMask false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_scaleMask
#define SC_USE_CLAMP_TO_BORDER_scaleMask false
#endif

#ifndef scaleMaskUV
#define scaleMaskUV 0
#endif
uniform NF_PRECISION		float pupilRadius;
uniform sampler2D albedoTex;
uniform mat3 albedoTexTransform;
uniform vec4 albedoTexUvMinMax;
uniform vec4 albedoTexBorderColor;
uniform vec4 albedoTexSize;

#ifndef SC_USE_UV_TRANSFORM_albedoTex
#define SC_USE_UV_TRANSFORM_albedoTex false
#define SC_SOFTWARE_WRAP_MODE_U_albedoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_albedoTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_albedoTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_albedoTex
#define SC_USE_CLAMP_TO_BORDER_albedoTex false
#endif

#ifndef albedoTexUV
#define albedoTexUV 0
#endif
uniform NF_PRECISION		float opacity;

#if defined(STUDIO)
uniform NF_PRECISION float2 Port_Scale_N046;
uniform NF_PRECISION float2 Port_Center_N046;
uniform NF_PRECISION float4 Port_Value_N035;
uniform NF_PRECISION float4 Port_Input0_N081;
uniform NF_PRECISION float3 Port_Albedo_N072;
uniform NF_PRECISION float Port_Opacity_N072;
uniform NF_PRECISION float2 Port_Scale_N069;
uniform NF_PRECISION float2 Port_Center_N069;
uniform NF_PRECISION float3 Port_Emissive_N072;
uniform NF_PRECISION float Port_Metallic_N072;
uniform NF_PRECISION float Port_Roughness_N072;
uniform NF_PRECISION float3 Port_AO_N072;
uniform NF_PRECISION float3 Port_SpecularAO_N072;
uniform NF_PRECISION float4 Port_Import_N074;
uniform NF_PRECISION float Port_Import_N075;
uniform NF_PRECISION float4 Port_Input1_N077;
uniform NF_PRECISION float Port_Import_N078;
uniform NF_PRECISION float Port_Import_N080;
uniform NF_PRECISION float2 Port_Scale_N015;
uniform NF_PRECISION float2 Port_Center_N015;
uniform NF_PRECISION float2 Port_Scale_N016;
uniform NF_PRECISION float2 Port_Center_N016;
uniform NF_PRECISION float Port_Input1_N031;
uniform NF_PRECISION float Port_Input2_N031;
uniform NF_PRECISION float Port_Value_N025;
uniform NF_PRECISION float Port_Input1_N024;
uniform NF_PRECISION float Port_Value_N029;
uniform NF_PRECISION float4 Port_Input1_N037;
uniform NF_PRECISION float4 Port_Input2_N037;
uniform NF_PRECISION float2 Port_Center_N032;
uniform NF_PRECISION float Port_Opacity_N006;
uniform NF_PRECISION float3 Port_Emissive_N006;
uniform NF_PRECISION float3 Port_AO_N006;
#endif

struct ssGlobals
{
	bool InVertexShader;
	bool InPixelShader;
	float2 Surface_UVCoord0;
	float3 VertexNormal_WorldSpace;
	float3 BumpedNormal;
	float3 VertexTangent_WorldSpace;
	float3 VertexBinormal_WorldSpace;
	float3 CameraPosition;
	float3 SurfacePosition_WorldSpace;
};

void Node61_Texture_2D_Object_Parameter(ssFunctionDefs)
{

}

void Node62_Surface_UV_Coord_0(out float2 UVCoord,ssFunctionDefs)
{
	UVCoord=Globals.Surface_UVCoord0;
}

void Node63_Surface_Normal(out float3 Normal,ssFunctionDefs)
{
	Normal=Globals.VertexNormal_WorldSpace;
}

void Node64_Offset_Coords(in float2 CoordsIn,in float2 Offset,out float2 CoordsOut,ssFunctionDefs)
{
	CoordsOut=CoordsIn+Offset;
}

void Node46_Scale_Coords(in float2 CoordsIn,out float2 CoordsOut,ssFunctionDefs)
{
	float2 Scale=ssPortConstant(float2(0.2,0.2),Port_Scale_N046);
	float2 Center=ssPortConstant(float2(0.5,0.6),Port_Center_N046);

	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node66_Texture_2D_Sample(in float2 UVCoord,out float4 Color,ssFunctionDefs)
{
		vec4 aoTexSample;
	SAMPLE_TEXTURE(aoTexSample,aoTex,UVCoord,SC_USE_UV_TRANSFORM_aoTex,aoTexTransform,SC_SOFTWARE_WRAP_MODE_U_aoTex,SC_SOFTWARE_WRAP_MODE_V_aoTex,SC_USE_UV_MIN_MAX_aoTex,aoTexUvMinMax,SC_USE_CLAMP_TO_BORDER_aoTex,aoTexBorderColor,0.0);;
	Color=aoTexSample;

	ssPreviewSave(Color,66,false)
}

void Node35_Color_Value(out float4 Output,ssFunctionDefs)
{
	float4 Value=ssPortConstant(float4(0.216953,0.153246,0.156024,1.0),Port_Value_N035);

	Output=Value;
}

void Node10_Add(in float4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0+Input1;
}

void Node68_Surface_UV_Coord_0(out float2 UVCoord,ssFunctionDefs)
{
	UVCoord=Globals.Surface_UVCoord0;
}

void Node69_Scale_Coords(in float2 CoordsIn,out float2 CoordsOut,ssFunctionDefs)
{
	float2 Scale=ssPortConstant(float2(1.4,1.4),Port_Scale_N069);
	float2 Center=ssPortConstant(float2(0.5,0.5),Port_Center_N069);

	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node26_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 normalOutSample;
	SAMPLE_TEXTURE(normalOutSample,normalOut,UVCoord,SC_USE_UV_TRANSFORM_normalOut,normalOutTransform,SC_SOFTWARE_WRAP_MODE_U_normalOut,SC_SOFTWARE_WRAP_MODE_V_normalOut,SC_USE_UV_MIN_MAX_normalOut,normalOutUvMinMax,SC_USE_CLAMP_TO_BORDER_normalOut,normalOutBorderColor,0.0);;
	Output=normalOutSample;

	Output.xyz=Output.xyz*(255.0/128.0) -1.0;
}

void Node72_PBR_Lighting(in float3 Normal,out float4 Output,ssFunctionDefs)
{
	float3 Albedo=ssPortConstant(float3(0.0,0.0,0.0),Port_Albedo_N072);
	float Opacity=ssPortConstant(float(0.0),Port_Opacity_N072);
	float3 Emissive=ssPortConstant(float3(0.0,0.0,0.0),Port_Emissive_N072);
	float Metallic=ssPortConstant(float(0.0),Port_Metallic_N072);
	float Roughness=ssPortConstant(float(0.0),Port_Roughness_N072);
	float3 AO=ssPortConstant(float3(1.0,1.0,1.0),Port_AO_N072);
	float3 SpecularAO=ssPortConstant(float3(1.0,1.0,1.0),Port_SpecularAO_N072);

	#if !defined(sc_ProjectiveShadowsCaster)

	Globals.BumpedNormal=float3x3(Globals.VertexTangent_WorldSpace,Globals.VertexBinormal_WorldSpace,Globals.VertexNormal_WorldSpace)*Normal;

	#endif

	ngsAlphaTest(Opacity);

	#if !defined(sc_ProjectiveShadowsCaster)

	Output=ngsCalculateLighting(Albedo,Opacity,Globals.BumpedNormal,Emissive,Metallic,Roughness,AO,SpecularAO);

	#else

	Output=float4(Albedo,Opacity);

	#endif

	Output=max(Output,0.0);
}

void Node74_Float_Import(in float4 Import,out float4 Value,ssFunctionDefs)
{
	Value=Import;
}

void Node75_Float_Import(out float Value,ssFunctionDefs)
{
	float Import=ssPortConstant(float(1.6),Port_Import_N075);

	Value=clamp(Import,0.0,5.0);
}

void Node76_Multiply(in float4 Input0,in float Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*float4(Input1);
}

void Node77_Dot_Product(in float4 Input0,out float Output,ssFunctionDefs)
{
	float4 Input1=ssPortConstant(float4(0.2125,0.7154,0.0721,0.0),Port_Input1_N077);

	Output=dot(Input0,Input1);
}

void Node78_Float_Import(out float Value,ssFunctionDefs)
{
	float Import=ssPortConstant(float(0.0),Port_Import_N078);

	Value=clamp(Import,0.0,2.0);
}

void Node79_Mix(in float Input0,in float4 Input1,in float Input2,out float4 Output,ssFunctionDefs)
{
	Output=mix(float4(Input0),Input1,float4(Input2));
}

void Node80_Float_Import(out float Value,ssFunctionDefs)
{
	float Import=ssPortConstant(float(1.6),Port_Import_N080);

	Value=clamp(Import,0.0,2.0);
}

void Node81_Mix(in float4 Input1,in float Input2,out float4 Output,ssFunctionDefs)
{
	float4 Input0=ssPortConstant(float4(0.5,0.5,0.5,0.0),Port_Input0_N081);

	Output=mix(Input0,Input1,float4(Input2));
}

void Node82_Float_Export(in float4 Value,out float4 Export,ssFunctionDefs)
{
	Export=Value;
}

void Node11_Surface_UV_Coord_0(out float2 UVCoord,ssFunctionDefs)
{
	UVCoord=Globals.Surface_UVCoord0;
}

void Node15_Scale_Coords(in float2 CoordsIn,out float2 CoordsOut,ssFunctionDefs)
{
	float2 Scale=ssPortConstant(float2(1.2,1.2),Port_Scale_N015);
	float2 Center=ssPortConstant(float2(0.5,0.5),Port_Center_N015);

	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node16_Scale_Coords(in float2 CoordsIn,out float2 CoordsOut,ssFunctionDefs)
{
	float2 Scale=ssPortConstant(float2(6.0,6.0),Port_Scale_N016);
	float2 Center=ssPortConstant(float2(0.5,0.5),Port_Center_N016);

	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node85_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 normalInSample;
	SAMPLE_TEXTURE(normalInSample,normalIn,UVCoord,SC_USE_UV_TRANSFORM_normalIn,normalInTransform,SC_SOFTWARE_WRAP_MODE_U_normalIn,SC_SOFTWARE_WRAP_MODE_V_normalIn,SC_USE_UV_MIN_MAX_normalIn,normalInUvMinMax,SC_USE_CLAMP_TO_BORDER_normalIn,normalInBorderColor,0.0);;
	Output=normalInSample;

	Output.xyz=Output.xyz*(255.0/128.0) -1.0;
}

void Node28_Swizzle(in float4 Input,out float Output,ssFunctionDefs)
{
	Output=Input.a;
}

void Node31_Clamp(in float Input0,out float Output,ssFunctionDefs)
{
	float Input1=ssPortConstant(float(0.0),Port_Input1_N031);
	float Input2=ssPortConstant(float(1.0),Port_Input2_N031);

	Output=clamp(Input0,Input1,Input2);
}

void Node25_Float_Value(out float Output,ssFunctionDefs)
{
	float Value=ssPortConstant(float(0.16),Port_Value_N025);

	Output=Value+0.001;
	Output -=0.001;
}

void Node14_Multiply(in float Input0,in float Input1,out float Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node22_Surface_Normal(out float3 Normal,ssFunctionDefs)
{
	Normal=Globals.VertexNormal_WorldSpace;
}

void Node18_View_Vector(out float3 ViewVector,ssFunctionDefs)
{
	ViewVector=normalize(Globals.CameraPosition -Globals.SurfacePosition_WorldSpace);
}

void Node24_Multiply(in float3 Input0,out float3 Output,ssFunctionDefs)
{
	float Input1=ssPortConstant(float(-1.0),Port_Input1_N024);

	Output=Input0*float3(Input1);
}

void Node27_Add(in float3 Input0,in float3 Input1,out float3 Output,ssFunctionDefs)
{
	Output=Input0+Input1;
}

void Node19_Multiply(in float Input0,in float3 Input1,out float3 Output,ssFunctionDefs)
{
	Output=float3(Input0)*Input1;

	ssPreviewSave(float4(Output.xyz,1.0),19,false)
}

void Node29_Float_Value(out float Output,ssFunctionDefs)
{
	float Value=ssPortConstant(float(5.0),Port_Value_N029);

	Output=Value+0.001;
	Output -=0.001;
}

void Node5_Multiply(in float2 Input0,in float Input1,out float2 Output,ssFunctionDefs)
{
	Output=Input0*float2(Input1);
}

void Node20_Offset_Coords(in float2 CoordsIn,in float2 Offset,out float2 CoordsOut,ssFunctionDefs)
{
	CoordsOut=CoordsIn+Offset;
}

void Node17_Offset_Coords(in float2 CoordsIn,in float2 Offset,out float2 CoordsOut,ssFunctionDefs)
{
	CoordsOut=CoordsIn+Offset;

	ssPreviewSave(float4(CoordsOut.xy,0.0,1.0),17,false)
}

void Node8_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 scaleMaskSample;
	SAMPLE_TEXTURE(scaleMaskSample,scaleMask,UVCoord,SC_USE_UV_TRANSFORM_scaleMask,scaleMaskTransform,SC_SOFTWARE_WRAP_MODE_U_scaleMask,SC_SOFTWARE_WRAP_MODE_V_scaleMask,SC_USE_UV_MIN_MAX_scaleMask,scaleMaskUvMinMax,SC_USE_CLAMP_TO_BORDER_scaleMask,scaleMaskBorderColor,0.0);;
	Output=scaleMaskSample;

	ssPreviewSave(Output,8,false)
}

void Node34_Float_Parameter(out float Output,ssFunctionDefs)
{
	Output=pupilRadius;
}

void Node38_One_Minus(in float Input0,out float Output,ssFunctionDefs)
{
	Output=1.0 -Input0;
}

void Node33_Add(in float4 Input0,in float Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0+float4(Input1);

	ssPreviewSave(Output,33,false)
}

void Node37_Clamp(in float4 Input0,out float4 Output,ssFunctionDefs)
{
	float4 Input1=ssPortConstant(float4(0.0,0.0,0.0,0.0),Port_Input1_N037);
	float4 Input2=ssPortConstant(float4(1.0,1.0,1.0,1.0),Port_Input2_N037);

	Output=clamp(Input0,Input1,Input2);
}

void Node32_Scale_Coords(in float2 CoordsIn,in float2 Scale,out float2 CoordsOut,ssFunctionDefs)
{
	float2 Center=ssPortConstant(float2(0.5,0.5),Port_Center_N032);

	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node30_Scale_Coords(in float2 CoordsIn,in float2 Scale,in float2 Center,out float2 CoordsOut,ssFunctionDefs)
{
	CoordsOut.xy=(CoordsIn.xy -Center)*Scale+Center;
}

void Node7_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 pupilTexSample;
	SAMPLE_TEXTURE(pupilTexSample,pupilTex,UVCoord,SC_USE_UV_TRANSFORM_pupilTex,pupilTexTransform,SC_SOFTWARE_WRAP_MODE_U_pupilTex,SC_SOFTWARE_WRAP_MODE_V_pupilTex,SC_USE_UV_MIN_MAX_pupilTex,pupilTexUvMinMax,SC_USE_CLAMP_TO_BORDER_pupilTex,pupilTexBorderColor,0.0);;
	Output=pupilTexSample;
}

void Node2_Texture_2D_Parameter(in float2 UVCoord,out float4 Output,ssFunctionDefs)
{
		vec4 albedoTexSample;
	SAMPLE_TEXTURE(albedoTexSample,albedoTex,UVCoord,SC_USE_UV_TRANSFORM_albedoTex,albedoTexTransform,SC_SOFTWARE_WRAP_MODE_U_albedoTex,SC_SOFTWARE_WRAP_MODE_V_albedoTex,SC_USE_UV_MIN_MAX_albedoTex,albedoTexUvMinMax,SC_USE_CLAMP_TO_BORDER_albedoTex,albedoTexBorderColor,0.0);;
	Output=albedoTexSample;
}

void Node21_Multiply(in float4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node6_PBR_Lighting(in float3 Albedo,in float3 Normal,out float4 Output,ssFunctionDefs)
{
	float Opacity=ssPortConstant(float(1.0),Port_Opacity_N006);
	float3 Emissive=ssPortConstant(float3(0.0,0.0,0.0),Port_Emissive_N006);
	float3 AO=ssPortConstant(float3(1.0,1.0,1.0),Port_AO_N006);

	#if !defined(sc_ProjectiveShadowsCaster)

	Globals.BumpedNormal=float3x3(Globals.VertexTangent_WorldSpace,Globals.VertexBinormal_WorldSpace,Globals.VertexNormal_WorldSpace)*Normal;

	#endif

	ngsAlphaTest(Opacity);

	Albedo=max(Albedo,0.0);

	#if !defined(sc_ProjectiveShadowsCaster)

	float Metallic=0.0;
	float Roughness=1.0;

	vec3 SpecularAO=vec3(1.0);

	Output=ngsCalculateLighting(Albedo,Opacity,Globals.BumpedNormal,Emissive,Metallic,Roughness,AO,SpecularAO);

	#else

	Output=float4(Albedo,Opacity);

	#endif

	Output=max(Output,0.0);
}

void Node60_Blend(in float4 Base,in float4 Color,out float4 Output,ssFunctionDefs)
{

	Output.rgb=vec3(max(Base.r,Color.r),max(Base.g,Color.g),max(Base.b,Color.b));
	Output.rgb=mix(Base.rgb,Output.rgb,Color.a);
	Output.a=Base.a;
}

void Node41_Multiply(in float4 Input0,in float4 Input1,out float4 Output,ssFunctionDefs)
{
	Output=Input0*Input1;
}

void Node1_Float_Parameter(out float Output,ssFunctionDefs)
{
	Output=opacity;
}

void Node92_Construct_Vector(in float3 Value1,in float Value2,out float4 Value,ssFunctionDefs)
{
	Value.xyz=Value1;
	Value.w=Value2;
}

#if !defined(RENDER_CONSTANT_COLOR)

void main()
{
	ssSetupPreview_Pixel

	float4 FinalColor=float4(1.0,1.0,1.0,1.0);

	ssGlobals Globals;
	Globals.InVertexShader			=false;
	Globals.InPixelShader			=true;
	Globals.Surface_UVCoord0		=varTex0;
	Globals.VertexNormal_WorldSpace	=normalize(varNormal);
	Globals.BumpedNormal			=float3(0.0);
	Globals.VertexTangent_WorldSpace=normalize(varTangent.xyz);
	Globals.VertexBinormal_WorldSpace=cross(Globals.VertexNormal_WorldSpace,Globals.VertexTangent_WorldSpace.xyz)*varBitangentSign;
	Globals.CameraPosition			=sc_Camera.position;
	Globals.SurfacePosition_WorldSpace=varPos;

	{
		Node61_Texture_2D_Object_Parameter(ssFunctionInputs);

		float2 UVCoord_N62;
		Node62_Surface_UV_Coord_0(UVCoord_N62,ssFunctionInputs);

		float3 Normal_N63;
		Node63_Surface_Normal(Normal_N63,ssFunctionInputs);

		float2 CoordsOut_N64;
		Node64_Offset_Coords(UVCoord_N62,Normal_N63.xy,CoordsOut_N64,ssFunctionInputs);

		float2 CoordsOut_N46;
		Node46_Scale_Coords(CoordsOut_N64,CoordsOut_N46,ssFunctionInputs);

		float4 Color_N66;
		Node66_Texture_2D_Sample(CoordsOut_N46,Color_N66,ssFunctionInputs);

		float4 Output_N35;
		Node35_Color_Value(Output_N35,ssFunctionInputs);

		float4 Output_N10;
		Node10_Add(Color_N66,Output_N35,Output_N10,ssFunctionInputs);

		float2 UVCoord_N68;
		Node68_Surface_UV_Coord_0(UVCoord_N68,ssFunctionInputs);

		float2 CoordsOut_N69;
		Node69_Scale_Coords(UVCoord_N68,CoordsOut_N69,ssFunctionInputs);

		float4 Output_N26;
		Node26_Texture_2D_Parameter(CoordsOut_N69,Output_N26,ssFunctionInputs);

		float4 Output_N72;
		Node72_PBR_Lighting(Output_N26.xyz,Output_N72,ssFunctionInputs);

		float4 Value_N74;
		Node74_Float_Import(Output_N72,Value_N74,ssFunctionInputs);

		float Value_N75;
		Node75_Float_Import(Value_N75,ssFunctionInputs);

		float4 Output_N76;
		Node76_Multiply(Value_N74,Value_N75,Output_N76,ssFunctionInputs);

		float Output_N77;
		Node77_Dot_Product(Output_N76,Output_N77,ssFunctionInputs);

		float Value_N78;
		Node78_Float_Import(Value_N78,ssFunctionInputs);

		float4 Output_N79;
		Node79_Mix(Output_N77,Output_N76,Value_N78,Output_N79,ssFunctionInputs);

		float Value_N80;
		Node80_Float_Import(Value_N80,ssFunctionInputs);

		float4 Output_N81;
		Node81_Mix(Output_N79,Value_N80,Output_N81,ssFunctionInputs);

		float4 Export_N82;
		Node82_Float_Export(Output_N81,Export_N82,ssFunctionInputs);

		float2 UVCoord_N11;
		Node11_Surface_UV_Coord_0(UVCoord_N11,ssFunctionInputs);

		float2 CoordsOut_N15;
		Node15_Scale_Coords(UVCoord_N11,CoordsOut_N15,ssFunctionInputs);

		float2 CoordsOut_N16;
		Node16_Scale_Coords(CoordsOut_N15,CoordsOut_N16,ssFunctionInputs);

		float4 Output_N85;
		Node85_Texture_2D_Parameter(CoordsOut_N15,Output_N85,ssFunctionInputs);

		float Output_N28;
		Node28_Swizzle(Output_N85,Output_N28,ssFunctionInputs);

		float Output_N31;
		Node31_Clamp(Output_N28,Output_N31,ssFunctionInputs);

		float Output_N25;
		Node25_Float_Value(Output_N25,ssFunctionInputs);

		float Output_N14;
		Node14_Multiply(Output_N31,Output_N25,Output_N14,ssFunctionInputs);

		float3 Normal_N22;
		Node22_Surface_Normal(Normal_N22,ssFunctionInputs);

		float3 ViewVector_N18;
		Node18_View_Vector(ViewVector_N18,ssFunctionInputs);

		float3 Output_N24;
		Node24_Multiply(ViewVector_N18,Output_N24,ssFunctionInputs);

		float3 Output_N27;
		Node27_Add(Normal_N22,Output_N24,Output_N27,ssFunctionInputs);

		float3 Output_N19;
		Node19_Multiply(Output_N14,Output_N27,Output_N19,ssFunctionInputs);

		float Output_N29;
		Node29_Float_Value(Output_N29,ssFunctionInputs);

		float2 Output_N5;
		Node5_Multiply(Output_N19.xy,Output_N29,Output_N5,ssFunctionInputs);

		float2 CoordsOut_N20;
		Node20_Offset_Coords(CoordsOut_N16,Output_N5,CoordsOut_N20,ssFunctionInputs);

		float2 CoordsOut_N17;
		Node17_Offset_Coords(CoordsOut_N15,Output_N19.xy,CoordsOut_N17,ssFunctionInputs);

		float4 Output_N8;
		Node8_Texture_2D_Parameter(CoordsOut_N17,Output_N8,ssFunctionInputs);

		float Output_N34;
		Node34_Float_Parameter(Output_N34,ssFunctionInputs);

		float Output_N38;
		Node38_One_Minus(Output_N34,Output_N38,ssFunctionInputs);

		float4 Output_N33;
		Node33_Add(Output_N8,Output_N38,Output_N33,ssFunctionInputs);

		float4 Output_N37;
		Node37_Clamp(Output_N33,Output_N37,ssFunctionInputs);

		float2 CoordsOut_N32;
		Node32_Scale_Coords(CoordsOut_N17,Output_N37.xy,CoordsOut_N32,ssFunctionInputs);

		float2 CoordsOut_N30;
		Node30_Scale_Coords(CoordsOut_N20,Output_N33.xy,CoordsOut_N32,CoordsOut_N30,ssFunctionInputs);

		float4 Output_N7;
		Node7_Texture_2D_Parameter(CoordsOut_N30,Output_N7,ssFunctionInputs);

		float4 Output_N2;
		Node2_Texture_2D_Parameter(CoordsOut_N32,Output_N2,ssFunctionInputs);

		float4 Output_N21;
		Node21_Multiply(Output_N7,Output_N2,Output_N21,ssFunctionInputs);

		float4 Output_N6;
		Node6_PBR_Lighting(Output_N21.xyz,Output_N85.xyz,Output_N6,ssFunctionInputs);

		float4 Output_N60;
		Node60_Blend(Export_N82,Output_N6,Output_N60,ssFunctionInputs);

		float4 Output_N41;
		Node41_Multiply(Output_N10,Output_N60,Output_N41,ssFunctionInputs);

		float Output_N1;
		Node1_Float_Parameter(Output_N1,ssFunctionInputs);

		float4 Value_N92;
		Node92_Construct_Vector(Output_N41.xyz,Output_N1,Value_N92,ssFunctionInputs);

		FinalColor=Value_N92;
	}

	FinalColor=ngsPixelShader(FinalColor);

	ssPreviewOutput_Pixel

	sc_FragData0=max(FinalColor,0.0);

	#if (defined(sc_OITCompositing4Layer) || defined(sc_OITCompositing8Layer) || defined(sc_OITDepthGather))
	processOIT(sc_FragData0);
	#endif

	#ifdef STUDIO
	vec4 Cost=getPixelRenderingCost();
	if (Cost.w > 0.0)
	sc_FragData0=Cost;
	#endif
}

#endif

#endif

#if sc_IsEditor
#error This is an exported shader. Please do not use shaders in Studio that have already been exported to a lens! Only use fresh shaders,presets,or shaders from existing Studio projects!
#endif

/// Exported with Lens Studio 3.3.3.0
